<?php
/**
* HELIUM V Synchronisation Modul for Magento 1.7.x
*
* Copyright (C) 2013 HELIUM V IT-Solutions GmbH
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU Affero General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* According to sec. 7 of the GNU Affero General Public License, version 3,
* the terms of the AGPL are supplemented with the following terms:
*
* "Helium V" and "Helium 5" are registered trademarks of HELIUM V IT-Solutions GmbH.
* The licensing of the Program under the AGPL does not imply a trademark license.
* Therefore any rights, title and interest in our trademarks remain entirely with us.
* If you want to propagate modified versions of the Program under the
* name "Helium V" or "Helium 5", you may only do so if you have a written permission
* by Helium V IT-Solutions GmbH (to acquire a permission please contact
* Helium V IT-Solutions at trademark@heliumv.com).
*      
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Affero General Public License for more details.
*
* You should have received a copy of the GNU Affero General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
* 
* @category Heliumv
* @package  Heliumv_Synchronization
* @author   Maximilian Hehenwarter <developers@heliumv.com>
* @license  http://www.gnu.org/licenses/  Affero General Public License (AGPL 3.0) 
*/

/**
 * Masterhelper beinhaltet alle zentralen Funktionen die in vielen anderen Klassen verwendet werden und an keine andere Stelle so richtig dazu gehören
 */
class Heliumv_Synchronization_Helper_Master extends Mage_Core_Helper_Abstract
{
    
    /** variablendeklaration logfile für order, zugriff mit self::$logfile_name */
    private static $logfile_name_ping="hv_ping.log";
    /** variablendeklaration logfile für master helper, zugriff mit self::$logfile_name */
    private static $logfile_name="hv_masterhelper.log";
    /** variablendeklaration logfile für error, zugriff mit self::$logfile_name */
    private static $logfile_name_error="hv_error_via_soap.log";
    
    /**
    * Liefert die Daten aus der Backend Magento Konfig für den Webservice Header
    */
    public static function getWebserviceHeaderData() 
    {
        return array(
            'isoCountry' => Mage::getStoreConfig('synchronization/heliumvconnect/country'), /* isoCountry */
            'isoLanguage' => Mage::getStoreConfig('synchronization/heliumvconnect/language'), /* isoLanguage */
            'shopName' => Mage::getStoreConfig('synchronization/heliumvconnect/shop_name'), /* shopName */
            'user' => Mage::getStoreConfig('synchronization/heliumvconnect/user'), /* user */
            'password' => Mage::getStoreConfig('synchronization/heliumvconnect/password'), /* password */
            'token' => Mage::getStoreConfig('synchronization/heliumvconnect/token') /* token */
        );
    }
    
    /**
    * Creates Soap Timeout Log
    * 
    * @param mixed $function - Name of the Function Call
    * @param mixed $text - optional aditional text, for example type
    */
    public static function createSoapTimeoutLog($function,$text="")
    {
        if(self::getDebugFlag())
        {
            $php_default_socket_timeout=(int)ini_get('default_socket_timeout');
            $magento_config_soap_timeout_req=(int)Mage::getStoreConfig('synchronization/heliumvconnect/soap_timeout_request');
            $magento_config_soap_timeout_resp=(int)Mage::getStoreConfig('synchronization/heliumvconnect/soap_timeout_response');
            mage::log("Timeout Auftretreten fuer >>".$function."<<, Zusatzinfo:>>".$text."<<. (PHP Sockettimeout=".$php_default_socket_timeout.", Soap Request/Response Timout=".$magento_config_soap_timeout_req."/".$magento_config_soap_timeout_resp.")",zend_log::ERR,"hv_soaptimeout.log");
        }   
    }
    
    /**
    * Liefert die aktuelle Url für Ordersync
    */
    public static function getWebserviceUrlOrder()
    {
        return self::getWebserviceUrl("order");    
    }
    
    /**
    * Liefert die aktuelle Url für Systemcalls, zB Ping
    */
    public static function getWebserviceUrlSystem()
    {
        return self::getWebserviceUrl("system");    
    }
    
    /**
    * Liefert die aktuelle Url für Artikelsync
    */
    public static function getWebserviceUrlArticle()
    {
        return self::getWebserviceUrl("article");    
    }
    /**
    * Liefert die aktuelle Url für Customersync
    */
    public static function getWebserviceUrlCustomer()
    {
        return self::getWebserviceUrl("customer");    
    }
    
    /**
    * Ermittelt die jeweilige WebserviceUrl abhängig von der Modulversion
    * @param mixed $type allowed types are: article, system, order
    * @return Url wenn OK, null im Fehlerfall
    */
    private static function getWebserviceUrl($type)
    {
        if($type!="article" && $type!="system" && $type!="order" && $type!="customer")
        {
            mage::log("Werservice Url mit Type:".$type." nicht gefunden",zend_log::ERR,self::$logfile_name);
            return null;    
        }
        
        try
        {
            //lies aktuelle installierte Modulversion von modul Heliumv_Synchronization aus
            $modul_version=self::getVersionModule();
            
            //lies hostname/port aus konfig
            $hostname=Mage::getStoreConfig('synchronization/heliumvconnect/server_hostname');
            $port=Mage::getStoreConfig('synchronization/heliumvconnect/server_port');
            
            //definiere string endung
            if($type=="article")
                $url_string="ArtikelFacBeanRest";
            if($type=="system")
                $url_string="SystemInfoFacBeanRest"; 
            if($type=="order")
                $url_string="AuftragFacBeanRest";
            if($type=="customer")
                $url_string="KundeFacBeanRest";      
            
            //berechne order urls pro magento tool version
            if (version_compare($modul_version, '0.0.3', '<=')===true)
            {
                if($type=="article")
                    $version_string="";
                if($type=="system")
                    $version_string=""; 
                if($type=="order")
                    $version_string="";
                if($type=="customer")
                    $version_string="";       
            }
            elseif(version_compare($modul_version, '0.0.4', '>=')===true)
            {
                if($type=="article")
                    $version_string="version2";
                if($type=="system")
                    $version_string="version1"; 
                if($type=="order")
                    $version_string="version1"; 
                if($type=="customer")
                    $version_string="version1";        
            }
            else
            {
                mage::log("ModulVersion:".$modul_version." hat aktuell noch keine Links zugewiesen, bitte in Heliumv_Synchronization_Helper_Master pflegen.",zend_log::ERR,self::$logfile_name);
                return null;    
            }
            
            //baue Url zusammen zB http://192.168.8.244:8080/lpserver-ejb/AuftragFacBeanRest?wsdl
            if($version_string!="")
                $full_url="http://".$hostname.":".$port."/lpserver-ejb/".$version_string."/".$url_string."?WSDL";
            else
                $full_url="http://".$hostname.":".$port."/lpserver-ejb/".$url_string."?WSDL";
                
            return $full_url;
        }
        catch(Exception $ex)
        {
            mage::log("Exception bei Ermittlung Webservice Url aufgetreten.".$ex->getMessage(),zend_log::ERR,self::$logfile_name);
            return null;
        }
    }
    
    /**
    * Liefert die aktuelle Modulversion aus der Einstellung in datei config.xml, vorsicht die muss bei buzild immer manuell nachgezigen werden
    */
    public static function getVersionModule()
    {
        return (string) Mage::getConfig()->getNode()->modules->Heliumv_Synchronization->version;
    }
    /**
    * Liefert die Mindestversion von HV-Build für die geweilige Modulversion
    */
    public static function getVersionHV()
    {
        $modul_version=self::getVersionModule();
        if (version_compare($modul_version, '0.0.6', '<=')===true)
            return "7543";
        elseif(version_compare($modul_version, '0.0.7', '>=')===true)
            return "7657";
        else
            return "unknown";
    }
    
    /**
    * Sendet generische Fehlermeldung an HV damit z.B. Stammdatenprobleme einfacher nachvollzogen werden können
    * Die SS ist Fire&Forget via SoapCall an HV
    * 
    * @param mixed $id - eindeutige Fehlernummer (z.B. Artikel 1xx, Category 2xx, Customer 3xx, Order 4xx, Undefined 9xx), im Fehlerfall 9999
    * @param mixed $text - spechender fehlertext, wird auf maximal 500 zeichen abgeschnitten
    * @param mixed $level - zend_log errorlevel, zB ERR=3, default=3
    * @param mixed $cat  - Error Kategory als text article, category, customer, order, undefined (default = undefined)
    * @param mixed $dataid - Helium ID (oder wenn nicht vorhanden Magento ID) des Datensatzes wo fehler aufgetreten ist, wird auf string maximal 50zeichen begrenzt
    * @param mixed $data - Fehlerhafte Daten, zB bei Stammdatenfehler, wird auf maximal 500 zeichen begrenzt
    * 
    * @return - im fehlerfall null, sonnst true
    */
    /*
    public static function old_sendErrorToHV($id,$text,$level=3,$cat="undefined",$dataid="",$data="")
    {
        $return=null;
        
        //variablen prüfen bzw limitieren/initialisieren
        $text=substr($text,0,500);
        $data=substr($data,0,500);
        $dataid=substr($dataid,0,50);
        if($cat!="article" && $cat!="category" && $cat!="customer" && $cat!="order")
            $cat="undefined";
        if((int)$level>7 || (int)$level<1)
            $level = 3;
        $id=(int)$id;
        if($id<1 || $id>9999)
            $id=9999;
        
        //im debugmodus meldung in logfile
        if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
            mage::log("ErrorID:".$id." - ErrorCat:".$cat." - ErrorText:".$text." - Data (".$dataid."):".$data,$level,self::$logfile_name_error);
        
        $options = array('trace' => 1);
        $server_url=Heliumv_Synchronization_Helper_Master::getWebserviceUrlSystem();
        try
        {
            //prevent running into a timeout
            set_time_limit(Mage::getStoreConfig('synchronization/heliumvconnect/soap_timeout'));
            //neuen soapclient aufmachen
            $client = new SoapClient($server_url,$options);
            //get item from the heliumv instance
            $result = $client->error(array('error' => array("category" => $cat, "dataId" => $dataid, "errorData" => $data, "id" => $id, "level" => $level, "text" => $text))); 

            //bei positiver rückmeldung
            if ($result->return->rc == 0)
            {                
                $return=true;
            }
            else
            {
                mage::log("getPricelistById - Keine Daten zu Preisliste:".$gid." gefunden oder RC<>0 bei Rückgabe. (RC=".$result->return->rc.")",zend_log::ERR,self::$logfile_name);
                $return=null;
            }
        }
        catch(SoapFault $exc)
        {
            mage::log("sendErrorToHV - ErrorID:".$id." mit DatenId:".$dataid." konnte nicht an HV uebermittelt werden - Exeption:".$exc->getMessage(),zend_log::ERR,self::$logfile_name);       
            $return=null;  
        }      
        
        $client=null;
        return $return;
          
    }
    */
    
    /**
    * Sendet generische Fehlermeldung an HV damit z.B. Stammdatenprobleme einfacher nachvollzogen werden können
    * Die SS ist Fire&Forget via SoapCall an HV unter Verwendung von NuSOAP als UTF-8 Datenstrom
    * 
    * @param mixed $id - eindeutige Fehlernummer (z.B. Artikel 1xx, Category 2xx, Customer 3xx, Order 4xx, Undefined 9xx), im Fehlerfall 9999
    * @param mixed $text - spechender fehlertext, wird auf maximal 500 zeichen abgeschnitten
    * @param mixed $level - zend_log errorlevel, zB ERR=3, default=3
    * @param mixed $cat  - Error Kategory als text article, category, customer, order, undefined (default = undefined)
    * @param mixed $dataid - Helium ID (oder wenn nicht vorhanden Magento ID) des Datensatzes wo fehler aufgetreten ist, wird auf string maximal 50zeichen begrenzt
    * @param mixed $data - Fehlerhafte Daten, zB bei Stammdatenfehler, wird auf maximal 500 zeichen begrenzt
    * 
    * @return - im fehlerfall null, sonnst true
    */
    public static function sendErrorToHV($id,$text,$level=3,$cat="undefined",$dataid="",$data="")
    {
        
        $return=null;
        
        //variablen prüfen bzw limitieren/initialisieren
        $text=substr($text,0,500);
        $data=substr($data,0,500);
        $dataid=substr($dataid,0,50);
        if($cat!="article" && $cat!="category" && $cat!="customer" && $cat!="order")
            $cat="undefined";
        if((int)$level>7 || (int)$level<1)
            $level = 3;
        $id=(int)$id;
        if($id<1 || $id>9999)
            $id=9999;
        
        //im debugmodus meldung auch in logfile
        if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
            mage::log("ErrorID:".$id." - ErrorCat:".$cat." - ErrorText:".$text." - Data (".$dataid."):".$data,$level,self::$logfile_name_error);
        
        //variablen setzen
        $options = array('trace' => 1);
        $login = Heliumv_Synchronization_Helper_Master::getWebserviceHeaderData();
        $server_url=Heliumv_Synchronization_Helper_Master::getWebserviceUrlSystem();
        $soap_timeout_request=Mage::getStoreConfig('synchronization/heliumvconnect/soap_timeout_request');
        $soap_timeout_response=Mage::getStoreConfig('synchronization/heliumvconnect/soap_timeout_response');
        $function_name=__FUNCTION__;
        $return=null;
        $nusoapnamespace="http://ejbfac.system.server.lp.com/";

        try
        {    
            //nusoap soapclient einbinden
            $file=Mage::getBaseDir('lib') . DIRECTORY_SEPARATOR.'nusoap-0.9.5'.DIRECTORY_SEPARATOR.'nusoap.php';
            include_once($file); //wird ab modul Heliumv_Synchronization-0.0.2 automatisch mit ausgeliefert
            
            //verbindung muss innerhalb von timeout stehen
            $nusoap = new nusoap_client($server_url,false,false, false, false, false, $soap_timeout_request, $soap_timeout_response);
            
            //change encoding settings -> soap client liefert bzw sendet utf8 -> bewirkt encoding="utf-8" in xml
            $nusoap->soap_defencoding = 'UTF-8';
            $nusoap->decode_utf8 = false;
            
            //fehler aufgetreten
            $nusoaperr = $nusoap->getError();
            if ($nusoaperr) {
                mage::log($function_name." Error - Nusoap Constructor error:".$nusoaperr,Zend_Log::ERR,self::$logfile_name_error);
                $return=null;
            }
            
            //sende error an HV
            $start_time=microtime(1);
            $nusoapresult = $nusoap->call('error',array('error' => array("category" => $cat, "dataId" => $dataid, "errorData" => $data, "id" => $id, "level" => $level, "text" => $text)),$nusoapnamespace);

            //prüfe auf fault
            if ($nusoap->fault) 
            {
                mage::log($function_name." Fault - NusoapCall Fault, Response:".print_r($nusoapresult,true),Zend_Log::ERR,self::$logfile_name_error);
                $return=null;
            } 
            else 
            {
                //prüfe auf fehler
                $nusoaperr = $nusoap->getError();
                if ($nusoaperr)
                {
                    //prüfe ob Timeout gekommen ist: server nicht erreichbar
                    if(substr($nusoaperr,0,53)=="HTTP Error: Couldn't open socket connection to server")
                    {
                        Heliumv_Synchronization_Helper_Master::createSoapTimeoutLog($function_name."_".$dataid,"Server nicht erreicht");
                        mage::log($function_name." Error - NusoapCall Error, Server nicht erreicht:".trim($nusoaperr),Zend_Log::ERR,self::$logfile_name_error);         
                    }
                    //prüfe ob Timeout gekommen ist:  erreichbar aber timeout
                    if(substr($nusoaperr,0,45)=="HTTP Error: socket read of headers timed out")
                    {
                        Heliumv_Synchronization_Helper_Master::createSoapTimeoutLog($function_name."_".$dataid,"Server erreicht aber Request kommt nicht zurueck");
                        mage::log($function_name." Error - NusoapCall Error, Server erreicht aber Request kommt nicht zurueck:".trim($nusoaperr),Zend_Log::ERR,self::$logfile_name_error);         
                    }
                    
                    $return=null;
                } 
                else
                {
                    //ermittle dauer rundtriptime für Call
                    $rtt=(microtime(1)-$start_time);  
                    
                    //alles ok ?
                    if(isset($nusoapresult["rc"]) && $nusoapresult["rc"]=="0")
                    {
                        //alles OK
                        if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                        {
                            mage::log($function_name." ErrorID:".$id." mit DatenId:".$dataid." in ".number_format($rtt,2)." Sek uebertragen.",zend_log::INFO,self::$logfile_name_error);
                        }
                        $return=$nusoapresult;
                    }
                    else
                    {
                        mage::log($function_name." ErrorID:".$id." mit DatenId:".$dataid." konnte nicht an HV uebermittelt werden.",Zend_Log::ERR,self::$logfile_name_error);    
                        $return=null;
                    }
                }
            } //else kein fault
        }
        catch(Exception $e)
        {
            mage::log($function_name." - Exeption:".$e->getMessage(),zend_log::ERR,self::$logfile_name_error);       
            $return=null; 
        }    
        
        //schreibe gesamten call in file wenn debug aktiv
        if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
        {
            Heliumv_Synchronization_Helper_Master::debugLog($function_name.'_'.date("His").'_request.txt',str_replace("><",">\n<",$nusoap->request),"no",gmdate("Ymd"));  
            Heliumv_Synchronization_Helper_Master::debugLog($function_name.'_'.date("His").'_response.txt',str_replace("><",">\n<",$nusoap->response),"no",gmdate("Ymd"));  
            
            //debug nur im fehlerfall schreiben da große datenmenge
            if($return==null)
                Heliumv_Synchronization_Helper_Master::debugLog($function_name.'_'.date("His").'_soapdebug.txt',$nusoap->debug_str,"no",gmdate("Ymd"));  
        }
        
        return $return ;
    }
    
    
    /**
    * Liefert die aktuell in der Config eingestellte Magento Store-ID zurück
    */
    public static function getStoreId()
    {
        $store_id=null;
        $store_id=Mage::getStoreConfig('synchronization/heliumvconnect/basic_store');
        if($store_id==null || !is_numeric($store_id) || $store_id=="")
        {
            mage::log("Store ID wird in Konfiguration zwingend benötigt, breche Verarbeitung ab.",zend_log::ALERT,self::$logfile_name);
            die("Store ID wird in Konfiguration zwingend benötigt, breche Verarbeitung ab.");
        }
        else
        {
            return $store_id;
        } 
    }
    

    
    
    
    
    
    /**
    * Liefert aufgrund des aktuell in der Config eingestellten Stores die Magento Website-ID zurück
    */
    public static function getWebsiteId()
    {
        $store_id=self::getStoreId();
        return Mage::getModel('core/store')->load($store_id)->getWebsiteId(); 
    }
    
    /**
    * Liefert die aktuell in der Config eingestellte Magento Store-Code zurück
    */
    public static function getStoreCode()
    {
        $store_id=self::getStoreId();
        return Mage::getModel('core/store')->load($store_id)->getCode();
    }
    
    /**
    * Liefert aufgrund des aktuell in der Config eingestellten Stores den Magento Website-Code zurück
    */
    public static function getWebsiteCode()
    {
        $store_id=self::getStoreId();
        return Mage::getModel('core/store')->load($store_id)->getWebsite()->getCode();
    }
    
    /**
    * Prüft die aktuell eingestellte StoreID, ist diese nicht korrekt wird ein log erstellt und die richtige gesetzt
    * @param mixed $create_log - optional, default=true (debug enabled), wenn auf false gesetzt wird kein log erstellt
    */
    public static function checkAndSetStoreId($create_log=true)
    {
        //check and set store
        $current_Store_id=Mage::app()->getStore()->getId();
        $store_from_config=self::getStoreId();
        
        //wenn storeids ungleich sind
        if((int)$current_Store_id!=(int)$store_from_config)
        {
            Mage::app()->setCurrentStore($store_from_config);
            if($create_log==true)
            {
                $backtrace = debug_backtrace();
                $function_name="unknown";
                if(isset($backtrace[1]['function']))
                    $function_name=$backtrace[1]['function'];
                
                if(self::getDebugFlag())
                    mage::log("checkAndSetStoreId - Funktion ".$function_name." hat StoreID von ".$current_Store_id." auf ".$store_from_config." gesetzt.",zend_log::WARN,self::$logfile_name);        
            }
        }
    }
    
    /**
    * Liefert Root Category für in Config eingestellten Store
    * @returns wenn alles OK die RootCategoryID, im Fehlerfall null
    */
    public static function getRootCategory()
    {
        $magento_parent_id=null;
        $store_code=self::getStoreCode();
        $magento_parent_id=Mage::getModel("core/store")->load($store_code)->getRootCategoryId();
        return $magento_parent_id;
    }
    
    /**
    * Liefert Debug Flag aus Backend Config
    * @return wenn debug aktiv dann 1 sonnst 0
    */
    public static function getDebugFlag()
    {
        $debug=null;
        $debug=Mage::getStoreConfig('synchronization/debug/debug_enabled');
        
        if(isset($debug) && $debug!="" && (int)$debug==1)
            return 1;
        else
            return 0;
    }    
    
    /**
    * Liefert positive log enabled Flag aus Backend Config (also soll wenn eine Verarbeitung für einen Artikel/Category etc OK ist ein Log geschrieben werden)
    * @return wenn positive log aktiv dann 1 sonnst 0
    */
    public static function getPositiveLogFlag()
    {
        $positive_log_enabled=null;
        $positive_log_enabled=Mage::getStoreConfig('synchronization/debug/positive_log_enabled');
        
        if(isset($positive_log_enabled) && $positive_log_enabled!="" && (int)$positive_log_enabled==1)
            return 1;
        else
            return 0;
    }
    
    /**
    * Liefert Async Flag (=Scheduler aktiv) aus Backend Config
    * @return wenn Async aktiv dann 1 sonnst 0
    */
    public static function getAsyncFlag()
    {
        $async=null;
        $async=Mage::getStoreConfig('synchronization/scheduler/scheduler_enabled');
        
        if(isset($async) && $async!="" && (int)$async==1)
            return 1;
        else
            return 0;
    }
    
    
    /**
    * Setze Ping Call an Helium.V ab, ist Helium nicht erreichbar wird nach erreichen des maximal counters ein Alarm eMail ausgelöst, sonnst wird counter zurück gesetzt
    * @returns Liefert 1=true wenn Helium erreichbar ist und 0=false wenn Helium nicht erreichbar ist
    */
    public static function doPingHVandSetCounter()
    {
        $return=false;
        $return=Heliumv_Synchronization_Helper_Master::doPingHV();
        
        if($return==true)
        {
            $error_counter=Mage::getStoreConfig('synchronization/ordertools/error_counter');
            if((int)$error_counter!=0)
            {
                //reset counter
                Mage::getConfig()->saveConfig('synchronization/ordertools/error_counter', 0 );
                Mage::getConfig()->reinit();
                Mage::app()->reinitStores();
                if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                    Mage::log("HV erreicht, Reset Ping Error Counter von ".$error_counter." auf 0",zend_log::DEBUG,self::$logfile_name_ping); 
            }   

        }
        else
        {
            //prüfe auf errorcounter
            $error_counter=Mage::getStoreConfig('synchronization/ordertools/error_counter');
            $max_error_counter=Mage::getStoreConfig('synchronization/ordertools/max_error_counter');
            
            //maximale fehleranzahl erreicht -> sende mail
            if($error_counter>=$max_error_counter && $error_counter!="" && $max_error_counter!="")
            {
                $subject="Error Magento orderSync";
                $message="Reached maximum Error Counts for Magento Order Sync";
                Heliumv_Synchronization_Helper_Master::sendErrorMail($subject,$message);    
                //reset counter
                Mage::getConfig()->saveConfig('synchronization/ordertools/error_counter', 0 );
                Mage::getConfig()->reinit();
                Mage::app()->reinitStores();
                Mage::log("Reset Ordersync Counter",null,self::$logfile_name_ping);    
            }
            else
            {
                //counter ++
                Mage::getConfig()->saveConfig('synchronization/ordertools/error_counter', (int)$error_counter+1 );
                Mage::getConfig()->reinit();
                Mage::app()->reinitStores();
                Mage::log("Error Counter auf ".((int)$error_counter+1)." gesetzt.",null,self::$logfile_name_ping);    
            }
        }
        
        return $return;
    }
    
    /**
    * Setze Ping Call an Helium.V ab, ist Helium nach 2sek request nicht erreichbar oder antwortet nach 4sek response nicht gilt es als nicht erreichbar 
    * @returns Liefert 1=true wenn Helium erreichbar ist und 0=false wenn Helium nicht erreichbar ist
    */
    public static function doPingHV()
    {
        //setze timeouts
        $req_timeout_sek=(int)Mage::getStoreConfig('synchronization/ordertools/ping_timeout');
        if($req_timeout_sek==null || (int)$req_timeout_sek<1 || $req_timeout_sek>10 )
            $req_timeout_sek=2;    
        $resp_timeout_sek=(int)$req_timeout_sek*2;
        
        $start_time=microtime(1);
        
        try
        {   
            //nusoap soapclient einbiunden
            $file=Mage::getBaseDir('lib') . DIRECTORY_SEPARATOR.'nusoap-0.9.5'.DIRECTORY_SEPARATOR.'nusoap.php';
            include_once($file); //wird ab modul Heliumv_Synchronization-0.0.2 automatisch mit ausgeliefert
            
            //server url aus cnfig lesen
            $server_url=Heliumv_Synchronization_Helper_Master::getWebserviceUrlSystem();
            
            //verbindung muss innerhalb 2sek stehen, response muss innerhalb 4 sek da sein
            $nusoap = new nusoap_client($server_url,false,false, false, false, false, $req_timeout_sek, $resp_timeout_sek);
            
            //change encoding settings -> soap client liefert utf8
            $nusoap->soap_defencoding='utf-8';
            $nusoap->decode_utf8 = false;
            
            //fehler aufgetreten
            $nusoaperr = $nusoap->getError();
            if ($nusoaperr) {
                mage::log("doPingHV Error - Nusoap Constructor error:".$nusoaperr,Zend_Log::ERR,self::$logfile_name_ping);
                return false;
            }

            //setze call ping ab
            //$nusoapresult = $nusoap->call('ping');
            $nusoapresult = $nusoap->call('ping',"","http://ejbfac.system.server.lp.com/");

            //prüfe auf fault
            if ($nusoap->fault) 
            {
                mage::log("doPingHV Fault - NusoapCall Fault, Response:".print_r($nusoapresult,true),Zend_Log::ERR,self::$logfile_name_ping);
                return false;
            } 
            else 
            {
                //prüfe auf fehler
                $nusoaperr = $nusoap->getError();
                if ($nusoaperr)
                {
                    //prüfe ob Timeout gekommen ist = erwarteter fehler
                    if(substr($nusoaperr,0,53)=="HTTP Error: Couldn't open socket connection to server")
                    {
                        mage::log("doPingHV - Timeout (req ".$req_timeout_sek."sek, Resp ".$resp_timeout_sek."sek)",Zend_Log::ERR,self::$logfile_name_ping);         
                    }
                    else
                    {
                        mage::log("doPingHV Error - NusoapCall Error:".$nusoaperr,Zend_Log::ERR,self::$logfile_name_ping);    
                    }
                    return false;
                } 
                else
                {
                    //alles ok ?
                    if(isset($nusoapresult["timestamp"]) && isset($nusoapresult["rc"]) && $nusoapresult["rc"]=="0")
                    {
                        //rundtriptime für erreichung war
                        $response_time=((double)$nusoapresult["timestamp"])/1000;
                        $rtt=abs($response_time-$start_time);
                        if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                            Mage::log("Zeitdifferenz fuer Ping zwischen Helium und Magento war ".number_format($rtt,4)." Sek",zend_log::DEBUG,self::$logfile_name_ping);      
                        
                        return true;
                    }
                    else
                    {
                        return false;
                    }
                }
            } //else kein fault
        } 
        catch(Exception $e)
        {
            mage::log("doPingHV  - Exception:".$e->getMessage(),Zend_Log::ERR,self::$logfile_name_ping); 
            return false;      
        }
    }
    
    /**
    * Schickt Error Mail wenn Probleme bei Order aufgetreten sind
    * to:eMailadresse muss im Backend eingestellt werden
    * from:eMailadresse ist nicht gesetzt
    * 
    * @param string $subject
    * @param string $message
    */
    public static function sendErrorMail($subject,$message)
    {
        $mail = Mage::getModel('core/email');
        //$mail->setToName('Your Name');
        $mail->setToEmail(mage::getStoreConfig("synchronization/ordertools/error_counter_mail"));
        $mail->setBody($message);
        $mail->setSubject($subject);
        //$mail->setFromEmail('donotreply@your-magento-sh0p.at');
        //$mail->setFromName("Msg to Show on Subject");
        $mail->setType('text');// YOu can use Html or text as Mail format
        try
        {
            mage::log("Send Error Mail Subject:".$subject." to ".mage::getStoreConfig("synchronization/ordertools/error_counter_mail"),zend_log::ERR,self::$logfile_name);
            $mail->send();
        }
        catch (Exception $exc)
        {
            mage::log("Error while sending Error-Mail Subject:".$subject." Message:".$exc->getMessage(),zend_log::ERR,self::$logfile_name);       
        }   
    }
    
    
    /**
    * Erstellt zusätzlich zum magento log im Verzeichniss /var/debug einträge direkt ohne ts
    * @param string $name
    * @param mixed $content
    * @param no/yes $addts soll zum filename ein zeitstempel voran gefügt werden, dann auf "yes" setzen
    * @param string $specialFolderName definiert Ordnername (zB gmdate("Ymd") oder Orders) in welchem die Dateien abgelegt werden
    */
    public static function debugLog($name,$content,$addts="no",$specialFolderName="")
    {
        //füge ts und spezialfoldername dazu
        if($addts=="no")
            if($specialFolderName!="")
                $file=mage::getBaseDir('var') .DIRECTORY_SEPARATOR.'debug'.DIRECTORY_SEPARATOR.$specialFolderName.DIRECTORY_SEPARATOR.$name;
            else
                $file=mage::getBaseDir('var') .DIRECTORY_SEPARATOR.'debug'.DIRECTORY_SEPARATOR.$name;    
        else
            if($specialFolderName!="")
                $file=mage::getBaseDir('var') .DIRECTORY_SEPARATOR.'debug'.DIRECTORY_SEPARATOR.$specialFolderName.DIRECTORY_SEPARATOR.gmdate("Ymd_His_").$name;
            else
                $file=mage::getBaseDir('var') .DIRECTORY_SEPARATOR.'debug'.DIRECTORY_SEPARATOR.gmdate("Ymd_His_").$name;
        
        //erstelle ordner wenn noch nicht da
        if(!file_exists(mage::getBaseDir('var') .DIRECTORY_SEPARATOR.'debug'))
            mkdir(mage::getBaseDir('var') .DIRECTORY_SEPARATOR.'debug');
        if($specialFolderName!="" && !file_exists(mage::getBaseDir('var') .DIRECTORY_SEPARATOR.'debug'.DIRECTORY_SEPARATOR.$specialFolderName))
            mkdir(mage::getBaseDir('var') .DIRECTORY_SEPARATOR.'debug'.DIRECTORY_SEPARATOR.$specialFolderName);
        
        //wenn datei bereits vorhanden -> füge random hinzu und prüfe erneut
        $counter=0;
        while(file_exists($file))
        {
            $file=$file."_rand".rand(10000,99999);
            $counter++;
            if($counter>100)
                break;
        }
            
        //für xml files verwende alte methode damit xmls direkt geöffnet werden können
        if(stristr($name,".xml"))
            $return =file_put_contents($file, print_r($content, true));
        else
            $return =file_put_contents($file, self::TVarDumper($content)); //vardumper unterstützt erkennung von rekursiven strukturen zur Vermeidung von Loops (bei den meisten Magento Objecten erforderlich)
            
        return $return;
    }
    



    /** Variable for TVarDumper */
    private static $_objects;
    /** Variable for TVarDumper */
    private static $_output;
    /** Variable for TVarDumper */
    private static $_depth;
    /**
     * Converts a variable into a string representation.
     * This method achieves the similar functionality as var_dump and print_r
     * but is more robust when handling complex objects such as PRADO controls.
     * siehe auch http://code.google.com/p/prado3/source/browse/tags/3.1.9/framework/Util/TVarDumper.php
     * @param mixed $var  variable to be dumped
     * @param integer $depth  maximum depth that the dumper should go into the variable. Defaults to 10.
     * @param mixed $highlight - not used
     * @return string the string representation of the variable
     */
    public static function TVarDumper($var,$depth=10,$highlight=false)
    {
            self::$_output='';
            self::$_objects=array();
            self::$_depth=$depth;
            self::dumpInternal($var,0);
            if($highlight)
            {
                    $result=highlight_string("<?php\n".self::$_output,true);
                    return preg_replace('/&lt;\\?php<br \\/>/','',$result,1);
            }
            else
                    return self::$_output;
    }
    /**
    * Converts a variable into a string representation, internal code
    * siehe auch http://code.google.com/p/prado3/source/browse/tags/3.1.9/framework/Util/TVarDumper.php
    * @param mixed $var
    * @param mixed $level
    */
    private static function dumpInternal($var,$level)
    {
            switch(gettype($var))
            {
                    case 'boolean':
                            self::$_output.=$var?'true':'false';
                            break;
                    case 'integer':
                            self::$_output.="$var";
                            break;
                    case 'double':
                            self::$_output.="$var";
                            break;
                    case 'string':
                            self::$_output.="'$var'";
                            break;
                    case 'resource':
                            self::$_output.='{resource}';
                            break;
                    case 'NULL':
                            self::$_output.="null";
                            break;
                    case 'unknown type':
                            self::$_output.='{unknown}';
                            break;
                    case 'array':
                            if(self::$_depth<=$level)
                                    self::$_output.='array(...)';
                            else if(empty($var))
                                    self::$_output.='array()';
                            else
                            {
                                    $keys=array_keys($var);
                                    $spaces=str_repeat(' ',$level*4);
                                    self::$_output.="array\n".$spaces.'(';
                                    foreach($keys as $key)
                                    {
                                            self::$_output.="\n".$spaces."    [$key] => ";
                                            self::$_output.=self::dumpInternal($var[$key],$level+1);
                                    }
                                    self::$_output.="\n".$spaces.')';
                            }
                            break;
                    case 'object':
                            if(($id=array_search($var,self::$_objects,true))!==false)
                                    self::$_output.=get_class($var).'#'.($id+1).'(...)';
                            else if(self::$_depth<=$level)
                                    self::$_output.=get_class($var).'(...)';
                            else
                            {
                                    $id=array_push(self::$_objects,$var);
                                    $className=get_class($var);
                                    $members=(array)$var;
                                    $keys=array_keys($members);
                                    $spaces=str_repeat(' ',$level*4);
                                    self::$_output.="$className#$id\n".$spaces.'(';
                                    foreach($keys as $key)
                                    {
                                            $keyDisplay=strtr(trim($key),array("\0"=>':'));
                                            self::$_output.="\n".$spaces."    [$keyDisplay] => ";
                                            self::$_output.=self::dumpInternal($members[$key],$level+1);
                                    }
                                    self::$_output.="\n".$spaces.')';
                            }
                            break;
            }
    }
    
    /**
    * Set Default attribute, behebt möglicherweise die sql exception magento general error 1452 cannot add or update a child row
    * verwendung mit $item->setAttributeSetId($this->getDefaultAttributeSetId('catalog/product'));     
    * idee von http://stackoverflow.com/questions/11759937/create-a-product-from-php-magento   
    * eventuelle alternative: setAttributeSetId($productModel->getResource()->getEntityType()->getDefaultAttributeSetId());
    * @param mixed $model_string
    */
    public static function getDefaultAttributeSetId($model_string="catalog/product")
    {
        $productModel = Mage::getModel($model_string);            
        return $productModel->getDefaultAttributeSetId();
    }
    
    /**
    * Prüft PHP Voraussetzungen und Unstimmigkeiten für Darstellung in einem Popup im Backend
    */
    public static function checkPHPVoraussetzungen()
    {
        //variablen initialisieren
        $string="";
        $php_default_socket_timeout=(int)ini_get('default_socket_timeout');
        $magento_soap_timeout_request=(int)Mage::getStoreConfig('synchronization/heliumvconnect/soap_timeout_request');
        $magento_soap_timeout_response=(int)Mage::getStoreConfig('synchronization/heliumvconnect/soap_timeout_response');
        
        //spezielle linebreaks wegen JS benötigt -> siehe http://bytes.com/topic/javascript/answers/145760-alert-box-line-breaks
        $string.="------------\\n";
        $string.="Check 1: PHP Socket Timeout > (SOAP Timeout Request + SOAP Timeout Response)\\n";
        if($php_default_socket_timeout>($magento_soap_timeout_request+$magento_soap_timeout_response))
            $string.=" -> OK: Socket-Timeout:".$php_default_socket_timeout.", Soap Timeout Request:".$magento_soap_timeout_request.", Soap Timeout Response:".$magento_soap_timeout_response."\\n";
        else
        {
            $string.=" -> ERROR: Socket-Timeout:".$php_default_socket_timeout.", Soap Timeout Request:".$magento_soap_timeout_request.", Soap Timeout Response:".$magento_soap_timeout_response."\\n";
            $string.=" -> Please change the default_socket_timeout in PHP.ini or the Soap Timeout in the Magento Backend -> HV Connector -> Basic Webservice Settings -> Soap Timeout\\n";
        }
        $string.="------------\\n";
        $string.="Versions:\\n";
        $string.="Magento Modul Heliumv_Synchronization Version: ".self::getVersionModule()." \\n";
        $string.="Needed HV Build Version min.: ".self::getVersionHV()." \\n";
        $string.="------------\\n"; 
        
        return $string;
    }
    
    /**
    * Helper für die Konvertierung Array to Object
    * 
    * @param mixed $array - array
    * @param stdClass $obj - obj ref
    * @return stdClass
    */
    private static function Helper_Array_to_Object_Converter($array, &$obj)
    {
        $obj=new stdClass();
        foreach ($array as $key => $value)
        {
            if (is_array($value))
            {
                $obj->$key = new stdClass();
                self::Helper_Array_to_Object_Converter($value, $obj->$key);
          }
          else
          {
          $obj->$key = $value;
          }
        }
        return $obj;
    }

    /**
    * Konvertiert ein Array in ein Object, Input von http://stackoverflow.com/questions/1869091/convert-array-to-object-php
    * @param mixed $array
    * @param mixed $add_return - wenn 1 dann wird automatisch auf oberster ebene das element return hinzugefügt (wird für unterschied nusoap, soapclient benötigt)
    * @return stdClass
    */
    public static function convertArrayToObject($array,$add_return=1)
    {
        $object= new stdClass();
        $object= self::Helper_Array_to_Object_Converter($array,$object);
        if($add_return==1)
        {
            $object2= new stdClass();
            $object2->return=$object;
            $object=null;
            return $object2;
        }
        else
        {
            return $object;    
        }
        
    }
    

}

?>