<?php
/**
* HELIUM V Synchronisation Modul for Magento 1.7.x
*
* Copyright (C) 2013 HELIUM V IT-Solutions GmbH
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU Affero General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* According to sec. 7 of the GNU Affero General Public License, version 3,
* the terms of the AGPL are supplemented with the following terms:
*
* "Helium V" and "Helium 5" are registered trademarks of HELIUM V IT-Solutions GmbH.
* The licensing of the Program under the AGPL does not imply a trademark license.
* Therefore any rights, title and interest in our trademarks remain entirely with us.
* If you want to propagate modified versions of the Program under the
* name "Helium V" or "Helium 5", you may only do so if you have a written permission
* by Helium V IT-Solutions GmbH (to acquire a permission please contact
* Helium V IT-Solutions at trademark@heliumv.com).
*      
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Affero General Public License for more details.
*
* You should have received a copy of the GNU Affero General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
* 
* @category Heliumv
* @package  Heliumv_Synchronization
* @author   Maximilian Hehenwarter <developers@heliumv.com>
* @license  http://www.gnu.org/licenses/  Affero General Public License (AGPL 3.0) 
*/

/**
* Hier werden alle Datenverarbeitungen und Helper für den Customer Sync abgelegt
*/
class Heliumv_Synchronization_Model_Customer extends Mage_Core_Model_Abstract
{
    /**
    * variablendeklaration logfile, zugriff mit self::$logfile_name
    */
    private static $logfile_name="hv_customersync.log";
    
    /**
    * Test function
    */
    public function test()
    {
        return "ja ich kann das customer modell jetzt aufrufen";
    }
    
    /**
    * Liefert MagentoAdressID und ExternalAdressID von Soap Response -> party
    * 
    * @param mixed $party_data - soap response von getCustomerDetail unter dem element party
    * @param mixed $type - erlaubt sind magento,external,group
    * @return - im Fehlerfall null und sonnst die jeweilige id
    */
    private function getAdressIDsFromXML($party_data,$type)
    {
        //falschen typ übergeben
        if($type!="magento" && $type!="external" && $type!="group" && $type!="email")
        {
            mage::log("getAdressIDsFromXML - Typ:".$type." nicht unterstützt",null,self::$logfile_name);
            return null;   
        }
        $customer_group=null;
        $helium_adress_id=null;
        $magento_adress_id=null;
        foreach($party_data->PARTY_ID as $party_id)
        {
            $attributes=$party_id->attributes();
            if($attributes["type"]=="heliumv")
            {
                $helium_adress_id=(string)$party_id;    
            }
            elseif($attributes["type"]=="customer_group")
            {
                $customer_group=(string)$party_id;      
            } 
            elseif($attributes["type"]=="customer_specific")
            {
                $magento_adress_id=(string)$party_id;          
            } 
            elseif($attributes["type"]=="customer_group_name")
            {
                //not used yet         
            }  
            else
            {
                mage::log("PartyID vom Typ: ".$attributes["type"]." wird noch nicht unterstützt",null,self::$logfile_name);
            } 
        }
        
        if($type=="magento")
            return $magento_adress_id;
        if($type=="external")
            return $helium_adress_id;
        if($type=="group")
            return $customer_group;
        if($type=="email")
        {
            if(isset($party_data->ADDRESS->EMAIL))
                return (string)$party_data->ADDRESS->EMAIL;
            else
                return null;
        }
    }
    
    /**
    * Hauptfunktion welche aufgerufen wird um Kundendaten mit HV zu syncronisieren
    * 
    * @param mixed $type - Fullsync mit typ=full, sync typ=update wenn nur die daten seit dem letzten zeitstempel geändert wurden
    */
    public function doCustomerSync($type)
    {
        //wenn Helium nicht erreichbar abbruch
        $ping=Heliumv_Synchronization_Helper_Master::doPingHV();
        if($ping!=1)
            return null;
        
        //speichere sync-start UTC Zeitstempel um ihn nachher bei erfolg zu setzen
        $ts_sync_start=gmdate("Y-m-d H:i:s");
        $abbruch_counter_max=Mage::getStoreConfig('synchronization/debug/max_sync_article'); //debugoption, bricht verarbeitung nach x kunden ab
        $abbruch_counter=0;
        
        //prüfe auf gültigen typ
        if($type!="update" && $type!="full")
        {
            mage::log("doCustomerSync Typ ".$type." wird nicht unterstützt, Abbruch.",zend_log::ERR,self::$logfile_name);
            return null;
        }   
        
        mage::log("########## doCustomerSync start - type:".$type.",async:".Heliumv_Synchronization_Helper_Master::getAsyncFlag()." #########",zend_log::INFO,self::$logfile_name);  
        
        //lese liste von kunden von hv aus
        $customer_list=$this->getCustomers($type);
        
        //sind daten vorhanden und call nicht abgestürzt etc
        if($customer_list!=null && isset($customer_list->return->customers->customer))
        {
            //hinweis auf async verarbeitung in log
            if(Heliumv_Synchronization_Helper_Master::getAsyncFlag())
                mage::log("Daten werden asyncron ueber Scheduler verarbeitet",zend_log::INFO,self::$logfile_name);
        
            //gehe durch alle Adressen und lese details aus und speichere sie dann in den adressen bzw. lege neue kudnen an
            foreach($customer_list->return->customers->customer as $customer)
            {
                $partner_id=$customer->id;
                
                //weiter nur wenn ID gefunden
                if($partner_id!=null)
                {
                
                    //starte abholen der details und speichern syncron oder asyncron
                    if(Heliumv_Synchronization_Helper_Master::getAsyncFlag())
                    {
                        $scheduler_obj=new Heliumv_Synchronization_Model_Scheduler();
                        $scheduler_obj->createSchedulerElement(get_class($this),"processCustomerDetails",array($partner_id),5);
                    }  
                    else
                    {
                        $magento_cid=$this->processCustomerDetails($partner_id);
                    } 
                
                    //breche verarbeitung ab wenn debug option maximale artikel/kategorien/customers eingestellt ist
                    if($abbruch_counter_max!="" && $abbruch_counter_max!=0 && $abbruch_counter>=$abbruch_counter_max)
                    {
                        mage::log("doCustomerSync Schleife abgebrochen weil Maximale Debug Kunden auf ".$abbruch_counter_max." gestellt waren.",zend_log::ERR,self::$logfile_name); 
                        //return null;
                        break; //aus schleife springen
                    }
                    else
                    {
                        $abbruch_counter++;    
                    }
                }
                
            } //foreach alle kunden
        
        } //if daten gefunden
        
        
        //wenn alles ok setze sync ts (bewusst bei beiden typen full und update gesetzt, gilt für artikel und categorien)
        Mage::getConfig()->saveConfig('synchronization/customersync/last_sync_ts', $ts_sync_start );
        Mage::getConfig()->reinit();
        Mage::app()->reinitStores();
        if(Heliumv_Synchronization_Helper_Master::getPositiveLogFlag())
            Mage::log("Setze Zeitstempel letzter doCustomerSync auf ".$ts_sync_start,zend_log::INFO,self::$logfile_name);   
        
        mage::log("########## doCustomerSync finished #########",zend_log::INFO,self::$logfile_name);  
        
    }
    
    /**
    * Verarbeitet Customer Details (Soapcall + Speicherung Customer/Adress), wird syncron und asyncron verwendet
    * 
    * @param mixed $partner_id - Helium Partner ID (ID eines eindeutigen Adresssatzes)
    * @return liefert magento customer/adress id zurück, oder null im fehlerfall
    */
    public function processCustomerDetails($partner_id)
    {
        $return = null;
        
        //partner details aus hv auslesen
        $customer_detail=$this->getCustomerDetails($partner_id);   
        
        //wenn keine daten vorhanden -> abbruch
        if(!isset($customer_detail->return->bmecatParties))
        {
            return null;
        }
        
        //gehe durch alle adressen=partys dieses kunden, typischerweise eine
        $bme_cat_info = simplexml_load_string($customer_detail->return->bmecatParties);
        
        foreach($bme_cat_info->PARTY as $party_data)
        {
            //lese ids ein bzw lege benötigte elemente an
            $external_aid = $this->getAdressIDsFromXML($party_data,"external");
            $magento_aid = $this->getAdressIDsFromXML($party_data,"magento");
            $external_group_id = $this->getAdressIDsFromXML($party_data,"group");
            $customer_email = $this->getAdressIDsFromXML($party_data,"email");
            $magento_aid_from_external_aid=$this->convertExternalAidToAdressId($external_aid);
            $magento_aid_from_adress_object=Mage::getModel('customer/address')->load($magento_aid)->getId();
            $magento_group_id=$this->checkCustomerGroup($external_group_id); //prüfe ob kundengruppoe exisitiert, sonnst lege kundengruppe an       
            $magento_obj_load_by_mail=mage::getModel("customer/customer")->setStoreId(Heliumv_Synchronization_Helper_Master::getStoreId())->setWebsiteId(Heliumv_Synchronization_Helper_Master::getWebsiteId())->loadByEmail($customer_email); //lade magento kunden by mail adresse
            
            //kann in magento eine adresse mit der internen oder externen id gefunden werden und adressobject ist vorhanden -> dann adresse aktualisieren
            if($magento_aid!=null && $magento_obj_load_by_mail->getId()!=null && $magento_aid_from_adress_object!=null)
            {
                //aktualisiere adresse wenn unterschiede festgestellt wurden
                $return = $this->saveCustomerAdress($magento_aid,$magento_group_id,$party_data);
            }
            elseif($magento_aid_from_external_aid!=null)
            {  
                //aktualisiere adresse wenn unterschiede festgestellt wurden
                $return = $this->saveCustomerAdress($magento_aid_from_external_aid,$magento_group_id,$party_data);      
            }
            elseif($magento_obj_load_by_mail->getId()!=null) //es gibt bereits einen kunden mit dieser emailadresse
            {
                //$return = "ERROR"; //liefert bewusst nicht NULL da sonnst ein Scheduler Retry stattfinden würde
                //mage::log("Es gibt bereits mit der eMailadresse:".$customer_email." den Kunden mit MagentoId:".$magento_obj_load_by_mail->getId().". Der neue Kunde mit External_Aid:".$external_aid." wurde nicht angelegt",zend_log::ERR,self::$logfile_name);    
                mage::log("processCustomerDetails - eMail:".$customer_email." gehoert zu Magento-KundenId:".$magento_obj_load_by_mail->getId().". Die Adresse mit ExternerID:".$external_aid." wird diesem Kunden hinzugefügt.",zend_log::WARN,self::$logfile_name);    
                
                //erstelle eine neue adresse für den kunden mit der selben emailadresse wenn adressid nicht gefunden wird da zB adresse oder Kunde gelöscht wurden
                $return = $this->saveCustomerAdress(null,$magento_group_id,$party_data,$magento_obj_load_by_mail->getId(),$external_aid);  
            }
            else
            {
                //erstelle neuen kunden, liefert KundenID, ERROR oder NULL zurück, Null=Schwerer Fehler -> Scheduler retry
                $return = $this->createCustomer($party_data,$external_aid,$magento_group_id);
            }
            
        } //foreach alle adressen eines datensatzes
        
        return $return;    
    }
    
    /**
    * Löscht alle aus HV importierten Kunden (konkret alle die eine external_cid haben)
    */
    public function deleteImportedCustomers()
    {
        //initialisierung
        $resource = Mage::getSingleton('core/resource');
        $db_read = $resource->getConnection('core_read');
        $count=0;
        
        //setzen von adminstore zwingend benötigt da sonnst löschen nicht möglich ist
        Mage::app()->setCurrentStore(Mage::getModel('core/store')->load(Mage_Core_Model_App::ADMIN_STORE_ID));

        //ermittle alle kundenids die zu löschen sind
        $sql="SELECT a.entity_id FROM ".$resource->getTableName("customer_entity_varchar")." a, ".$resource->getTableName("eav_attribute")." b where b.attribute_code='external_cid' and a.entity_type_id=b.entity_type_id and a.attribute_id=b.attribute_id and a.value is not null and a.value<>''";
        $customers = $db_read->fetchCol($sql);
        
        foreach ($customers as $customer_id) {
            try {
                Mage::getModel('customer/customer')->load($customer_id)->delete();
                $count++;
            } catch (Exception $ex) {
                Mage::log("Fehler beim Löschen des Kunden:".$customer_id.", Exception:".$ex->getMessage(),zend_LOG::ERR,self::$logfile_name);
            }
        }

        mage::log("deleteImportedCustomers - Es wurden manuell ".$count." Kunden gelöscht.",zend_log::INFO,self::$logfile_name);
        
        //wieder rücksetzen auf normalen store
        Heliumv_Synchronization_Helper_Master::checkAndSetStoreId(false);
    }
    
    
    /**
    * Liefert alle Kunden IDs die in den Webshop exportiert werden sollen via SoapCall getCustomers aus HV
    * @param $type full für alle daten, update für alle daten seit ts
    * @return SoapResult oder im Fehlerfall null
    */
    private function old_getCustomers($type)
    {
        $time = microtime(true);
        
        //prüfe auf gültigen typ
        if($type!="update" && $type!="full")
        {
            mage::log("getCustomers Typ (".$type.") wird nicht unterstützt, Abbruch.",zend_log::ERR,self::$logfile_name);
            return null;
        }  
        
        //ermittle letztgültigen sync zeitstempel
        if($type=="update")
        {
            $ts_sync=gmdate("Y-m-d H:i:s"); //initialwert utc ts setzen
            $ts_last_sync=Mage::getStoreConfig('synchronization/customersync/last_sync_ts');
            if(isset($ts_last_sync) and $ts_last_sync!="" and strtotime($ts_last_sync)>0)
            {
                $ts_sync=$ts_last_sync;  
            }            
            mage::log("Verwende Zeitstempel letzter doCustomerSync ".$ts_sync,null,self::$logfile_name);
        }
        
        
        
        $options = array('trace' => 1);
        $login = Heliumv_Synchronization_Helper_Master::getWebserviceHeaderData();
        $server_url=Heliumv_Synchronization_Helper_Master::getWebserviceUrlCustomer();
        $return=null;
        
        try
        {
            //prevent running into a timeout
            set_time_limit(Mage::getStoreConfig('synchronization/heliumvconnect/soap_timeout'));
            
            //neuen soapclient aufmachen
            $client = new SoapClient($server_url,$options);
            
            //get items from the heliumv instance
            if($type=="full")
                $result = $client->getCustomers(array('arg0' => $login)); 
            if($type=="update")
                $result = $client->getCustomersChanged(array('arg0' => $login, 'changedDateTime' => $ts_sync)); 
                
            //bei positiver rückmeldung
            if ($result->return->rc == 0 && isset($result->return->customers))
            {                
                $return= $result;
                if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                {
                    //ermittle anzahl der elemente
                    if(isset($result->return->customers->customer))
                        $count=count($result->return->customers->customer);
                    else
                        $count=0;
                    
                    if($type=="update")
                        mage::log("getCustomersChanged hat ".$count." Kunden gefunden, Dauer: ".round(microtime(true) - $time, 2)."Sek",zend_log::INFO,self::$logfile_name);
                    else
                        mage::log("getCustomers hat ".$count." Kunden gefunden, Dauer: ".round(microtime(true) - $time, 2)."Sek",zend_log::INFO,self::$logfile_name);
                }
            }
            else
            {
                mage::log("getCustomers mit Typ=".$type." - Keine Kunden gefunden oder RC<>0 bei Rückgabe. (RC=".$result->return->rc.", ItemAnzahl=".count($result->return->items).", Typ=".$type.")",zend_log::WARN,self::$logfile_name);
                $return=null;
            }
        }
        catch(SoapFault $exc)
        {
            mage::log("getCustomers mit Typ=".$type." - Exeption:".$exc,zend_log::ERR,self::$logfile_name);       
            $return=null; 
        }    
        
        //wenn debug aktiv schreibe request und response in debug
        if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
        {
            Heliumv_Synchronization_Helper_Master::debugLog("getCustomers_".$type."_request.xml",$client->__getLastRequest() ,"no",gmdate("Ymd"));
            Heliumv_Synchronization_Helper_Master::debugLog("getCustomers_".$type."_response.xml",$client->__getLastResponse() ,"no",gmdate("Ymd"));
        }
        
        $client=null; //soap client object deaktivieren
        return $return ;
    }
    
     /**
    * Liefert alle Kunden IDs die in den Webshop exportiert werden sollen via SoapCall getCustomers aus HV
    * @param $type full für alle daten, update für alle daten seit ts
    * @return SoapResult oder im Fehlerfall null
    */
    private function getCustomers($type)
    {
        $options = array('trace' => 1);
        $login = Heliumv_Synchronization_Helper_Master::getWebserviceHeaderData();
        $server_url=Heliumv_Synchronization_Helper_Master::getWebserviceUrlCustomer();
        $soap_timeout_request=Mage::getStoreConfig('synchronization/heliumvconnect/soap_timeout_request');
        $soap_timeout_response=Mage::getStoreConfig('synchronization/heliumvconnect/soap_timeout_response');
        $function_name=__FUNCTION__;
        $return=null;
        $nusoapnamespace="http://ejbfac.partner.server.lp.com/";
        
        //prüfe auf gültigen typ
        if($type!="update" && $type!="full")
        {
            mage::log($function_name." Typ (".$type.") wird nicht unterstützt, Abbruch.",zend_log::ERR,self::$logfile_name);
            return null;
        }  
        
        //ermittle letztgültigen sync zeitstempel
        if($type=="update")
        {
            $ts_sync=gmdate("Y-m-d H:i:s"); //initialwert utc ts setzen
            $ts_last_sync=Mage::getStoreConfig('synchronization/customersync/last_sync_ts');
            if(isset($ts_last_sync) and $ts_last_sync!="" and strtotime($ts_last_sync)>0)
            {
                $ts_sync=$ts_last_sync;  
            }            
            mage::log($function_name." Verwende Zeitstempel letzter CustomerSync ".$ts_sync,null,self::$logfile_name);
        }
        
        try
        {    
            //nusoap soapclient einbinden
            $file=Mage::getBaseDir('lib') . DIRECTORY_SEPARATOR.'nusoap-0.9.5'.DIRECTORY_SEPARATOR.'nusoap.php';
            include_once($file); //wird ab modul Heliumv_Synchronization-0.0.2 automatisch mit ausgeliefert
            
            //verbindung muss innerhalb von timeout stehen
            $nusoap = new nusoap_client($server_url,false,false, false, false, false, $soap_timeout_request, $soap_timeout_response);
            
            //change encoding settings -> soap client liefert utf8
            $nusoap->soap_defencoding='utf-8';
            $nusoap->decode_utf8 = false;
            
            //fehler aufgetreten
            $nusoaperr = $nusoap->getError();
            if ($nusoaperr) {
                mage::log($function_name." Error - Nusoap Constructor error:".$nusoaperr,Zend_Log::ERR,self::$logfile_name);
                $return=null;
            }

            //get shopgroups from the heliumv instance
            $start_time=microtime(1);
            if($type=="update")
            {
                $nusoapresult = $nusoap->call('getCustomersChanged',array('arg0' => $login,'changedDateTime' => $ts_sync),$nusoapnamespace);
            }
            if($type=="full")
            {
                $nusoapresult = $nusoap->call('getCustomers',array('arg0' => $login),$nusoapnamespace);
            }

            //prüfe auf fault
            if ($nusoap->fault) 
            {
                mage::log($function_name." Fault - NusoapCall Fault, Response:".print_r($nusoapresult,true),Zend_Log::ERR,self::$logfile_name);
                $return=null;
            } 
            else 
            {
                //prüfe auf fehler
                $nusoaperr = $nusoap->getError();
                if ($nusoaperr)
                {
                    //prüfe ob Timeout gekommen ist: server nicht erreichbar
                    if(substr($nusoaperr,0,53)=="HTTP Error: Couldn't open socket connection to server")
                    {
                        Heliumv_Synchronization_Helper_Master::createSoapTimeoutLog($function_name."_".$type,"Server nicht erreicht");
                        mage::log($function_name." Error - NusoapCall Error, Server nicht erreicht:".trim($nusoaperr),Zend_Log::ERR,self::$logfile_name);         
                    }
                    //prüfe ob Timeout gekommen ist:  erreichbar aber timeout
                    if(substr($nusoaperr,0,45)=="HTTP Error: socket read of headers timed out")
                    {
                        Heliumv_Synchronization_Helper_Master::createSoapTimeoutLog($function_name."_".$type,"Server erreicht aber Request kommt nicht zurueck");
                        mage::log($function_name." Error - NusoapCall Error, Server erreicht aber Request kommt nicht zurueck:".trim($nusoaperr),Zend_Log::ERR,self::$logfile_name);         
                    }
                    
                    $return=null;
                } 
                else
                {
                    //ermittle anzahl
                    $count=0;
                    if(isset($nusoapresult['customers']['customer']))
                        $count=(int)count($nusoapresult['customers']['customer']);
                    
                    //ermittle dauer rundtriptime für Call
                    $rtt=(microtime(1)-$start_time);  
                    
                    //alles ok ?
                    if(isset($nusoapresult["rc"]) && $nusoapresult["rc"]=="0" && $count!=0)
                    {
                        //alles OK
                        if(Heliumv_Synchronization_Helper_Master::getPositiveLogFlag())
                        {
                            mage::log($function_name." SoapCall lieferte ".$count." Elemente in ".number_format($rtt,2)." Sek zurueck.",zend_log::INFO,self::$logfile_name);
                        }
                        $return=$nusoapresult;
                    }
                    else
                    {
                        if($type=="full")
                            mage::log($function_name." Keine Kunden gefunden (ItemAnzahl=".$count.",Typ=".$type.") oder Problem bei RC=(".$nusoapresult["rc"].") aufgetreten.",Zend_Log::ERR,self::$logfile_name);    
                        else
                            mage::log($function_name." Keine Kunden gefunden (ItemAnzahl=".$count.",Typ=".$type.") oder Problem bei RC=(".$nusoapresult["rc"].") aufgetreten.",Zend_Log::WARN,self::$logfile_name);    
                        $return=null;
                    }
                }
            } //else kein fault
        }
        catch(Exception $e)
        {
            mage::log($function_name." - Exeption:".$e->getMessage(),zend_log::ERR,self::$logfile_name);       
            $return=null; 
        }    
        
        //schreibe gesamten call in file wenn debug aktiv
        if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
        {
            Heliumv_Synchronization_Helper_Master::debugLog($function_name.'_'.date("His").'_request.txt',str_replace("><",">\n<",$nusoap->request),"no",gmdate("Ymd"));  
            Heliumv_Synchronization_Helper_Master::debugLog($function_name.'_'.date("His").'_response.txt',str_replace("><",">\n<",$nusoap->response),"no",gmdate("Ymd"));  
            
            //debug nur im fehlerfall schreiben da große datenmenge
            if($return==null)
                Heliumv_Synchronization_Helper_Master::debugLog($function_name.'_'.date("His").'_soapdebug.txt',$nusoap->debug_str,"no",gmdate("Ymd"));  
        }
        
        //convertiere array zu object (nusoap liefert daten standardmäßig als array, in programmen wird structur aber alls object abgefragt)
        if($return!=null)
            $return=Heliumv_Synchronization_Helper_Master::convertArrayToObject($return);
        
        return $return ;
    }
    
    /**
    * Liest Details zu Kunden aus HV via Soapcall getCustomerById
    * @param mixed $aid HV Partner id (entspricht in magento dann der external adress id)
    * @return liefert soap-result oder im fehlerfall null zurück
    */
    private function old_getCustomerDetails($aid)
    {
        $options = array('trace' => 1);
        $login = Heliumv_Synchronization_Helper_Master::getWebserviceHeaderData();
        $server_url=Heliumv_Synchronization_Helper_Master::getWebserviceUrlCustomer();
        $return=null;
        
        try
        {
            //prevent running into a timeout
            set_time_limit(Mage::getStoreConfig('synchronization/heliumvconnect/soap_timeout'));
            //neuen soapclient aufmachen
            $client = new SoapClient($server_url,$options);
            //get item from the heliumv instance
            $result = $client->getCustomerById(array('arg0' => $login,'customerId' => $aid)); 

            //bei positiver rückmeldung
            if ($result->return->rc == 0 && isset($result->return->bmecatParties))
            {                
                $return=$result;
            }
            else
            {
                mage::log("getCustomerDetails - Keine Daten zu Kunden ".$aid." gefunden oder RC<>0 bei Rückgabe. (RC=".$result->return->rc.")",zend_log::WARN,self::$logfile_name);
                $return=null;
            }
        }
        catch(SoapFault $exc)
        {
            mage::log("getCustomerDetails Kunde:".$aid." - Exeption:".$exc->getMessage(),zend_log::ERR,self::$logfile_name);       
            $return=null;  
        }      
        
        //wenn debug aktiv schreibe request und response in debug
        if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
        {
            Heliumv_Synchronization_Helper_Master::debugLog("getCustomerDetails_".$aid."_request.xml",$client->__getLastRequest() ,"no",gmdate("Ymd"));
            Heliumv_Synchronization_Helper_Master::debugLog("getCustomerDetails_".$aid."_response.xml",$client->__getLastResponse() ,"no",gmdate("Ymd"));
        }
        
        $client=null;
        return $return;
    }
    
    /**
    * Liest Details zu Kunden aus HV via Soapcall getCustomerById
    * @param mixed $aid HV Partner id (entspricht in magento dann der external adress id)
    * @return liefert soap-result oder im fehlerfall null zurück
    */
    private function getCustomerDetails($aid)
    {
        $options = array('trace' => 1);
        $login = Heliumv_Synchronization_Helper_Master::getWebserviceHeaderData();
        $server_url=Heliumv_Synchronization_Helper_Master::getWebserviceUrlCustomer();
        $soap_timeout_request=Mage::getStoreConfig('synchronization/heliumvconnect/soap_timeout_request');
        $soap_timeout_response=Mage::getStoreConfig('synchronization/heliumvconnect/soap_timeout_response');
        $function_name=__FUNCTION__;
        $return=null;
        $nusoapnamespace="http://ejbfac.partner.server.lp.com/";
        
        try
        {    
            //nusoap soapclient einbinden
            $file=Mage::getBaseDir('lib') . DIRECTORY_SEPARATOR.'nusoap-0.9.5'.DIRECTORY_SEPARATOR.'nusoap.php';
            include_once($file); //wird ab modul Heliumv_Synchronization-0.0.2 automatisch mit ausgeliefert
            
            //verbindung muss innerhalb von timeout stehen
            $nusoap = new nusoap_client($server_url,false,false, false, false, false, $soap_timeout_request, $soap_timeout_response);
            
            //change encoding settings -> soap client liefert utf8
            $nusoap->soap_defencoding='utf-8';
            $nusoap->decode_utf8 = false;
            
            //fehler aufgetreten
            $nusoaperr = $nusoap->getError();
            if ($nusoaperr) {
                mage::log($function_name." Error - Nusoap Constructor error:".$nusoaperr,Zend_Log::ERR,self::$logfile_name);
                $return=null;
            }
            
            //get partner details from the heliumv instance
            $start_time=microtime(1);
            $nusoapresult = $nusoap->call('getCustomerById',array('arg0' => $login,'customerId' => $aid),$nusoapnamespace);

            //prüfe auf fault
            if ($nusoap->fault) 
            {
                mage::log($function_name." Fault - NusoapCall Fault, Response:".print_r($nusoapresult,true),Zend_Log::ERR,self::$logfile_name);
                $return=null;
            } 
            else 
            {
                //prüfe auf fehler
                $nusoaperr = $nusoap->getError();
                if ($nusoaperr)
                {
                    //prüfe ob Timeout gekommen ist: server nicht erreichbar
                    if(substr($nusoaperr,0,53)=="HTTP Error: Couldn't open socket connection to server")
                    {
                        Heliumv_Synchronization_Helper_Master::createSoapTimeoutLog($function_name."_".$aid,"Server nicht erreicht");
                        mage::log($function_name." Error - NusoapCall Error, Server nicht erreicht:".trim($nusoaperr),Zend_Log::ERR,self::$logfile_name);         
                    }
                    //prüfe ob Timeout gekommen ist:  erreichbar aber timeout
                    if(substr($nusoaperr,0,45)=="HTTP Error: socket read of headers timed out")
                    {
                        Heliumv_Synchronization_Helper_Master::createSoapTimeoutLog($function_name."_".$aid,"Server erreicht aber Request kommt nicht zurueck");
                        mage::log($function_name." Error - NusoapCall Error, Server erreicht aber Request kommt nicht zurueck:".trim($nusoaperr),Zend_Log::ERR,self::$logfile_name);         
                    }
                    
                    $return=null;
                } 
                else
                {
                    //ermittle dauer rundtriptime für Call
                    $rtt=(microtime(1)-$start_time);  
                    
                    //alles ok ?
                    if(isset($nusoapresult["rc"]) && $nusoapresult["rc"]=="0" && $nusoapresult['bmecatParties'])
                    {
                        //alles OK
                        if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                        {
                            mage::log($function_name." SoapCall lieferte Daten zu Kunde ".$aid." in ".number_format($rtt,2)." Sek zurueck.",zend_log::INFO,self::$logfile_name);
                        }
                        $return=$nusoapresult;
                    }
                    else
                    {
                        mage::log($function_name." Keine Daten zu Kunde ".$aid." gefunden oder Problem bei RC=(".$nusoapresult["rc"].") aufgetreten.",Zend_Log::ERR,self::$logfile_name);    
                        $return=null;
                    }
                }
            } //else kein fault
        }
        catch(Exception $e)
        {
            mage::log($function_name." - Exeption:".$e->getMessage(),zend_log::ERR,self::$logfile_name);       
            $return=null; 
        }    
        
        //schreibe gesamten call in file wenn debug aktiv
        if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
        {
            Heliumv_Synchronization_Helper_Master::debugLog($function_name.'_'.$aid.'_'.date("His").'_request.txt',str_replace("><",">\n<",$nusoap->request),"no",gmdate("Ymd"));  
            Heliumv_Synchronization_Helper_Master::debugLog($function_name.'_'.$aid.'_'.date("His").'_response.txt',str_replace("><",">\n<",$nusoap->response),"no",gmdate("Ymd"));  
            
            //debug nur im fehlerfall schreiben da große datenmenge
            if($return==null)
                Heliumv_Synchronization_Helper_Master::debugLog($function_name.'_'.$aid.'_'.date("His").'_soapdebug.txt',$nusoap->debug_str,"no",gmdate("Ymd"));  
        }
        
        //convertiere array zu object (nusoap liefert daten standardmäßig als array, in programmen wird structur aber alls object abgefragt)
        if($return!=null)
            $return=Heliumv_Synchronization_Helper_Master::convertArrayToObject($return);
        
        return $return ;
    }
    
    /**
    * Liest Details zu Kundengruppe (=Helium Preisliste) aus HV via Soapcall getPricelistById aus
    * @param mixed $gid - entspricht magento kundengruppe bzw. helium preislisten id
    * @return liefert Name der Kundengruppe oder im fehlerfall null zurück
    */
    private function old_getCustomerGroupName($gid)
    {
        $options = array('trace' => 1);
        $login = Heliumv_Synchronization_Helper_Master::getWebserviceHeaderData();
        $server_url=Heliumv_Synchronization_Helper_Master::getWebserviceUrlCustomer();
        $return=null;
        
        try
        {
            //prevent running into a timeout
            set_time_limit(Mage::getStoreConfig('synchronization/heliumvconnect/soap_timeout'));
            //neuen soapclient aufmachen
            $client = new SoapClient($server_url,$options);
            //get item from the heliumv instance
            $result = $client->getPricelistById(array('arg0' => $login,'pricelistId' => $gid)); 

            //bei positiver rückmeldung
            if ($result->return->rc == 0 && isset($result->return->pricelist->cnr))
            {                
                $return=$result->return->pricelist->cnr;
            }
            else
            {
                mage::log("getPricelistById - Keine Daten zu Preisliste:".$gid." gefunden oder RC<>0 bei Rückgabe. (RC=".$result->return->rc.")",zend_log::ERR,self::$logfile_name);
                $return=null;
            }
        }
        catch(SoapFault $exc)
        {
            mage::log("getPricelistById Preisliste:".$gid." - Exeption:".$exc->getMessage(),zend_log::ERR,self::$logfile_name);       
            $return=null;  
        }      
        
        //wenn debug aktiv schreibe request und response in debug
        if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
        {
            Heliumv_Synchronization_Helper_Master::debugLog("getPricelistById_".$gid."_request.xml",$client->__getLastRequest() ,"no",gmdate("Ymd"));
            Heliumv_Synchronization_Helper_Master::debugLog("getPricelistById_".$gid."_response.xml",$client->__getLastResponse() ,"no",gmdate("Ymd"));
        }
        
        $client=null;
        return $return;
    }
    
    /**
    * Liest Details zu Kundengruppe (=Helium Preisliste) aus HV via Soapcall getPricelistById aus (NuSOAP Call)
    * @param mixed $gid - entspricht magento kundengruppe bzw. helium preislisten id
    * @return liefert Name der Kundengruppe oder im fehlerfall null zurück
    */
    private function getCustomerGroupName($gid)
    {
        $options = array('trace' => 1);
        $login = Heliumv_Synchronization_Helper_Master::getWebserviceHeaderData();
        $server_url=Heliumv_Synchronization_Helper_Master::getWebserviceUrlCustomer();
        $soap_timeout_request=Mage::getStoreConfig('synchronization/heliumvconnect/soap_timeout_request');
        $soap_timeout_response=Mage::getStoreConfig('synchronization/heliumvconnect/soap_timeout_response');
        $function_name=__FUNCTION__;
        $return=null;
        $nusoapnamespace="http://ejbfac.partner.server.lp.com/";

        try
        {    
            //nusoap soapclient einbinden
            $file=Mage::getBaseDir('lib') . DIRECTORY_SEPARATOR.'nusoap-0.9.5'.DIRECTORY_SEPARATOR.'nusoap.php';
            include_once($file); //wird ab modul Heliumv_Synchronization-0.0.2 automatisch mit ausgeliefert
            
            //verbindung muss innerhalb von timeout stehen
            $nusoap = new nusoap_client($server_url,false,false, false, false, false, $soap_timeout_request, $soap_timeout_response);
            
            //change encoding settings -> soap client liefert utf8
            $nusoap->soap_defencoding='utf-8';
            $nusoap->decode_utf8 = false;
            
            //fehler aufgetreten
            $nusoaperr = $nusoap->getError();
            if ($nusoaperr) {
                mage::log($function_name." Error - Nusoap Constructor error:".$nusoaperr,Zend_Log::ERR,self::$logfile_name);
                $return=null;
            }
            
            //get artikel details from the heliumv instance
            $start_time=microtime(1);
            $nusoapresult = $nusoap->call('getPricelistById',array('arg0' => $login,'pricelistId' => $gid),$nusoapnamespace);

            //prüfe auf fault
            if ($nusoap->fault) 
            {
                mage::log($function_name." Fault - NusoapCall Fault, Response:".print_r($nusoapresult,true),Zend_Log::ERR,self::$logfile_name);
                $return=null;
            } 
            else 
            {
                //prüfe auf fehler
                $nusoaperr = $nusoap->getError();
                if ($nusoaperr)
                {
                    //prüfe ob Timeout gekommen ist: server nicht erreichbar
                    if(substr($nusoaperr,0,53)=="HTTP Error: Couldn't open socket connection to server")
                    {
                        Heliumv_Synchronization_Helper_Master::createSoapTimeoutLog($function_name."_".$gid,"Server nicht erreicht");
                        mage::log($function_name." Error - NusoapCall Error, Server nicht erreicht:".trim($nusoaperr),Zend_Log::ERR,self::$logfile_name);         
                    }
                    //prüfe ob Timeout gekommen ist:  erreichbar aber timeout
                    if(substr($nusoaperr,0,45)=="HTTP Error: socket read of headers timed out")
                    {
                        Heliumv_Synchronization_Helper_Master::createSoapTimeoutLog($function_name."_".$gid,"Server erreicht aber Request kommt nicht zurueck");
                        mage::log($function_name." Error - NusoapCall Error, Server erreicht aber Request kommt nicht zurueck:".trim($nusoaperr),Zend_Log::ERR,self::$logfile_name);         
                    }
                    
                    $return=null;
                } 
                else
                {
                    //ermittle dauer rundtriptime für Call
                    $rtt=(microtime(1)-$start_time);  
                    
                    //alles ok ?
                    if(isset($nusoapresult["rc"]) && $nusoapresult["rc"]=="0" && $nusoapresult['pricelist']['cnr'])
                    {
                        //alles OK
                        if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                        {
                            mage::log($function_name." SoapCall lieferte Daten zu Preisliste ".$gid." in ".number_format($rtt,2)." Sek zurueck.",zend_log::INFO,self::$logfile_name);
                        }
                        $return=$nusoapresult;
                    }
                    else
                    {
                        mage::log($function_name." Keine Daten zu Preisliste ".$gid." gefunden oder Problem bei RC=(".$nusoapresult["rc"].") aufgetreten.",Zend_Log::ERR,self::$logfile_name);    
                        $return=null;
                    }
                }
            } //else kein fault
        }
        catch(Exception $e)
        {
            mage::log($function_name." - Exeption:".$e->getMessage(),zend_log::ERR,self::$logfile_name);       
            $return=null; 
        }    
        
        //schreibe gesamten call in file wenn debug aktiv
        if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
        {
            Heliumv_Synchronization_Helper_Master::debugLog($function_name.'_'.date("His").'_request.txt',str_replace("><",">\n<",$nusoap->request),"no",gmdate("Ymd"));  
            Heliumv_Synchronization_Helper_Master::debugLog($function_name.'_'.date("His").'_response.txt',str_replace("><",">\n<",$nusoap->response),"no",gmdate("Ymd"));  
            
            //debug nur im fehlerfall schreiben da große datenmenge
            if($return==null)
                Heliumv_Synchronization_Helper_Master::debugLog($function_name.'_'.date("His").'_soapdebug.txt',$nusoap->debug_str,"no",gmdate("Ymd"));  
        }
        
        //convertiere array zu object (nusoap liefert daten standardmäßig als array, in programmen wird structur aber alls object abgefragt)
        if($return!=null)
            $return=Heliumv_Synchronization_Helper_Master::convertArrayToObject($return);
        
        return $return ;
    }
    
    
    /**
    * Erstellt einen neuen Customer inklusive einer Adresse über das Import FRramework AvS
    * 
    * @param mixed $customer_detail_party - Response SoapCall getCustomerDetails einer PARTY
    * @param mixed $external_aid - externe adressid
    * @param mixed $group_id  - magento kundengruppen id
    * @return Magento Customer ID, NULL im Schweren Fehlerfall und "ERROR" im leichten Fehlerfall (Scheduler retry nur bei Schwerer Fehlern)
    */
    private function createCustomer($customer_detail_party,$external_aid,$group_id)
    {
        //prüfe ob die wichtigsten pflichtfelder vorhanden sind -> sonnst alles beim speichern geprüft
        $errortext="";
        $abbruch=0;
        if((string)$customer_detail_party->ADDRESS->EMAIL==null || (string)$customer_detail_party->ADDRESS->EMAIL=="" || !Zend_Validate::is((string)$customer_detail_party->ADDRESS->EMAIL, 'EmailAddress'))
        {
            $errortext.=" Kunde ".$external_aid." hat keine gueltige eMail (".(string)$customer_detail_party->ADDRESS->EMAIL."), ";
            $abbruch=1;    
        }
        if((string)$customer_detail_party->ADDRESS->PHONE==null || (string)$customer_detail_party->ADDRESS->PHONE=="")
        {
            $errortext.=" Kunde ".$external_aid." hat keine Telefon -> Daten werden mit 00 initialisiert, ";
        }
        if((string)$customer_detail_party->ADDRESS->STREET==null || (string)$customer_detail_party->ADDRESS->STREET=="")
        {
            $errortext.=" Kunde ".$external_aid." hat keine Strasse, ";
            $abbruch=1; 
        }
        if((string)$customer_detail_party->ADDRESS->ZIP==null || (string)$customer_detail_party->ADDRESS->ZIP=="")
        {
            $errortext.=" Kunde ".$external_aid." hat keine Postleitzahl, ";
            $abbruch=1; 
        }
        if((string)$customer_detail_party->ADDRESS->CITY==null || (string)$customer_detail_party->ADDRESS->CITY=="")
        {
            $errortext.=" Kunde ".$external_aid." hat keine Stadt, ";
            $abbruch=1; 
        }
        if((string)$customer_detail_party->ADDRESS->CONTACT_DETAILS->CONTACT_NAME==null || (string)$customer_detail_party->ADDRESS->CONTACT_DETAILS->CONTACT_NAME=="")
        {
            $errortext.=" Kunde ".$external_aid." hat keinen Nachnamen, ";
            $abbruch=1; 
        }
        if((string)$customer_detail_party->ADDRESS->CONTACT_DETAILS->FIRST_NAME==null || (string)$customer_detail_party->ADDRESS->CONTACT_DETAILS->FIRST_NAME=="")
        {
            $errortext.=" Kunde ".$external_aid." hat keinen Vornamen, ";
            $abbruch=1; 
        }
            
        //schreibe error in log
        if($abbruch==1)
        {
            //melde Fehler auch nach Helium
            Heliumv_Synchronization_Helper_Master::sendErrorToHV(301,"Verarbeitung von PartnerID:".$external_aid." wegen Stammdatenproblemen abgebrochen",zend_log::ERR,"customer",$external_aid,$errortext);
            
            mage::log("Verarbeitung von Kunden ".$external_aid." abgebrochen, Stammdatenprobleme waren (".$errortext.")",zend_log::ERR,self::$logfile_name);
            return "ERROR";
        }
        elseif($errortext!="")
        {
            mage::log("Probleme mit Stammdaten von Kunden ".$external_aid." aufgetreten (".$errortext.")",zend_log::WARN,self::$logfile_name);
        }
        
        //passwortlänge einlesen
        $password_lenght=Mage::getStoreConfig('synchronization/customersync/init_password_lenght');
        if($password_lenght==null || $password_lenght=="" || (int)$password_lenght<5)
            $password_lenght=7; //default im fehlerfall erzwingen
       
        //gender verarbeiten / convertieren
        $gender=null;   
        if(isset($customer_detail_party->ADDRESS->CONTACT_DETAILS->TITLE))
        {
            if((string)$customer_detail_party->ADDRESS->CONTACT_DETAILS->TITLE=="Frau")
            {
                $gender="Female";   
            }
            elseif((string)$customer_detail_party->ADDRESS->CONTACT_DETAILS->TITLE=="Herr")
            {
                $gender="Male";        
            }
            else
            {
                $gender=null;    
            }
        }

        
        
        $data = array();

        $data[]=
        array(
            "email" => (string)$customer_detail_party->ADDRESS->EMAIL , //mandatory  - unique identifier
            "_website" => Heliumv_Synchronization_Helper_Master::getWebsiteCode() , //mandatory
            "_store" => Heliumv_Synchronization_Helper_Master::getStoreCode() ,
            "confirmation" => "" ,
            "created_at" => "" ,
            "created_in" => "" ,
            "disable_auto_group_change" => "" ,
            "dob" => "" ,
            "external_cid" => "hv" ,   //hv customer id -> id wird nicht gesetzt aber kennzeichen hv, damit man importierte und selbst angelegte kunden danach unterscheiden kann
            "firstname" => (string)$customer_detail_party->ADDRESS->CONTACT_DETAILS->FIRST_NAME , 
            "gender" => $gender ,  //Male / Female
            "group_id" => $group_id , //mandatory
            "lastname" => (string)$customer_detail_party->ADDRESS->CONTACT_DETAILS->CONTACT_NAME ,
            "middlename" => "" ,
            "password_hash" => "" ,
            "prefix" => (string)$customer_detail_party->ADDRESS->CONTACT_DETAILS->TITLE , //Mann,Frau,Firma
            "rp_token" => "" ,
            "rp_token_created_at" => "" ,
            "store_id" => "" ,
            "suffix" => "" ,
            "taxvat" => (string)$customer_detail_party->ADDRESS->VAT_ID ,
            "website_id" => Heliumv_Synchronization_Helper_Master::getWebsiteId() , 
            "password" => Mage::getModel('customer/customer')->generatePassword($password_lenght) , 
            "_address_city" => (string)$customer_detail_party->ADDRESS->CITY ,
            "_address_company" => (string)$customer_detail_party->ADDRESS->NAME ,   
            "_address_country_id" => (string)$customer_detail_party->ADDRESS->COUNTRY_CODED ,
            "_address_external_aid" => $external_aid , //hv adress id
            "_address_fax" => (string)$customer_detail_party->ADDRESS->FAX , 
            "_address_firstname" => (string)$customer_detail_party->ADDRESS->CONTACT_DETAILS->FIRST_NAME , 
            "_address_lastname" => (string)$customer_detail_party->ADDRESS->CONTACT_DETAILS->CONTACT_NAME ,  
            "_address_middlename" => "" ,
            "_address_postcode" => (string)$customer_detail_party->ADDRESS->ZIP ,
            "_address_prefix" => (string)$customer_detail_party->ADDRESS->CONTACT_DETAILS->TITLE ,
            "_address_region" => "" ,
            "_address_street" => (string)$customer_detail_party->ADDRESS->STREET ,
            "_address_suffix" => "" ,
            "_address_telephone" => (string)$customer_detail_party->ADDRESS->PHONE ? (string)$customer_detail_party->ADDRESS->PHONE : "00"  , //falls telefonnummer nicht gesetzt ist schreibe 00 als wert
            "_address_vat_id" => (string)$customer_detail_party->ADDRESS->VAT_ID ,
            "_address_default_billing_" => "" ,
            "_address_default_shipping_" => "" 
            );

        
        $time = microtime(true);
        //mage::log("Start Import Kunde ",null,self::$logfile_name);
        try {
            if(!file_exists(mage::getBaseDir("media").DIRECTORY_SEPARATOR."import"))
                mkdir(mage::getBaseDir("media").DIRECTORY_SEPARATOR."import");
            
            /** @var $import AvS_FastSimpleImport_Model_Import */
            $import = Mage::getModel('fastsimpleimport/import');
            $import
                ->setPartialIndexing(true)
                ->setBehavior(Mage_ImportExport_Model_Import::BEHAVIOR_APPEND)
                ->processCustomerImport($data);
            if(Heliumv_Synchronization_Helper_Master::getPositiveLogFlag())
                mage::log("Import Kunde (".(string)$customer_detail_party->ADDRESS->NAME.") Dauer:" . round(microtime(true) - $time, 2) . "s OK.",zend_log::INFO,self::$logfile_name);
            $return="OK";
        } catch (Exception $e) {
            mage::log("Exception Import Kunde (".(string)$customer_detail_party->ADDRESS->NAME."):".print_r($import->getErrorMessages(),1)." - ".$e->getMessage()." trace:\n".$e->getTraceAsString(),zend_log::ERR,self::$logfile_name);
            $return= null;
        }
        
        //send welcome mail to customer when setting enabled, input from http://stackoverflow.com/questions/13581576/how-to-send-new-password-from-magento-csv-import-of-customers
        if($return=="OK")
        {
            $cust = Mage::getModel('customer/customer');
            //hier nicht sicher ob das so stimmt
            $cust->setWebsiteId(Heliumv_Synchronization_Helper_Master::getWebsiteId());
            $cust->loadByEmail((string)$customer_detail_party->ADDRESS->EMAIL);
            //lese id des kunden für return aus
            $return = $cust->getId();
            
            //wenn mail senden aktiv sende welcome mail
            $send_init_mail=Mage::getStoreConfig('synchronization/customersync/send_initial_pwd');
            if(isset($send_init_mail) && $send_init_mail!="" && (int)$send_init_mail==1)
            {
                //übernehme klartextpwd von oben, für mail text benötigt
                $cust->setData("password",$data[0]["password"]);
                $cust->sendNewAccountEmail();
            }
            
        }
        
        return $return;
        
    }
    
    /**
    * Vergleicht bestehende Adresse mit neuen Adressdaten und aktualisiert Customer Adresse bei Unterschieden
    * @param mixed $magento_aid - Magento Adsress ID
    * @param mixed $magento_group_id - ID Magento Kundengruppe laut XML
    * @param mixed $party_data - Response SoapCall getCustomerDetails einer einzelnen party
    * @param mixed $customer_id - optional kann auch die KundenID mit übergeben werden falls die Adresse dem Kunden noch nicht zugewiesen ist
    * @param mixed $external_aid - optional kann auch die externe adressID mit übergeben werden falls die Adresse dem Kunden noch nicht zugewiesen ist
    * @return Magento Adress ID oder null im schweren Fehlerfall
    */
    private function saveCustomerAdress($magento_aid,$magento_group_id,$party_data,$customer_id=null,$external_aid=null)
    {
        //lade bestehende adresse
        $address = Mage::getModel('customer/address')->load($magento_aid);
        $customer_group_id=null;
        
        //prüfe ob adresse geladen werden konnte, wenn OK lade Kundengruppe aus der Adresse zugehörigem Customer
        if($address->getId()!=null && (int)$address->getId()==(int)$magento_aid)
        {
            $customer_group_id = Mage::getModel('customer/customer')->load($address->getParentId())->getGroupId();    
        }
        
        //wenn sich die kundengruppe geändert hat diese am customerobj speichern
        if($magento_group_id!=null && $customer_group_id!=null && (int)$magento_group_id!=(int)$customer_group_id)
        {
            $customer=Mage::getModel('customer/customer')->load($address->getParentId());
            $customer->setGroupId($magento_group_id);
            try
            {
                $customer->save();
                mage::log("Kundengruppe/Preisliste von Customer mit MagentoID:".$customer->getId()." von ".$customer_group_id." auf ".$magento_group_id." geändert (jeweils Magento GroupIDs)",zend_log::INFO,self::$logfile_name);      
            }
            catch(Exception $e)
            {
                mage::log("Exception bei Kundengruppenaenderung von CustomerID:".$customer->getId()." aufgetreten - ".$e->getMessage(),zend_log::ERR,self::$logfile_name);      
            }
        }
        
        
        //vergleiche alle object werte mit xml
        $unterschied=0;
        $errortext="";
        $return=$magento_aid;     
        if($address->getFirstname()!=(string)$party_data->ADDRESS->CONTACT_DETAILS->FIRST_NAME)  
        {
            $unterschied=1;
            $errortext.=" Unterschied bei Vornamen (".$address->getFirstname().") vs. (".(string)$party_data->ADDRESS->CONTACT_DETAILS->FIRST_NAME.") gefunden, ";
        }
        if($address->getLastname()!=(string)$party_data->ADDRESS->CONTACT_DETAILS->CONTACT_NAME)   
        {
            $unterschied=1;
            $errortext.=" Unterschied bei Nachnamen (".$address->getLastname().") vs. (".(string)$party_data->ADDRESS->CONTACT_DETAILS->CONTACT_NAME.") gefunden, ";
        }
        if($address->getPrefix()!=(string)$party_data->ADDRESS->CONTACT_DETAILS->TITLE)   
        {
            $unterschied=1;
            $errortext.=" Unterschied bei Prefix/Titel (".$address->getPrefix().") vs. (".(string)$party_data->ADDRESS->CONTACT_DETAILS->TITLE.") gefunden, ";
        }
        if($address->getCompany()!=(string)$party_data->ADDRESS->NAME)  
        {
            $unterschied=1;
            $errortext.=" Unterschied bei Firmennamen (".$address->getCompany().") vs. (".(string)$party_data->ADDRESS->NAME.") gefunden, ";
        }
        if(current($address->getStreet())!=(string)$party_data->ADDRESS->STREET)   
        {
            $unterschied=1;
            $errortext.=" Unterschied bei Strasse (".current($address->getStreet()).") vs. (".(string)$party_data->ADDRESS->STREET.") gefunden, ";
        }
        if($address->getPostcode()!=(string)$party_data->ADDRESS->ZIP) 
        {
            $unterschied=1;
            $errortext.=" Unterschied bei PLZ (".$address->getPostcode().") vs. (".(string)$party_data->ADDRESS->ZIP.") gefunden, ";
        }
        if($address->getCity()!=(string)$party_data->ADDRESS->CITY) 
        {
            $unterschied=1;
            $errortext.=" Unterschied bei Stadt (".$address->getCity().") vs. (".(string)$party_data->ADDRESS->CITY.") gefunden, ";
        }
        if($address->getCountryId()!=(string)$party_data->ADDRESS->COUNTRY_CODED) 
        {
            $unterschied=1;
            $errortext.=" Unterschied bei Land (".$address->getCountryId().") vs. (".(string)$party_data->ADDRESS->COUNTRY_CODED.") gefunden, ";
        }
        if($address->getTelephone()!=(string)$party_data->ADDRESS->PHONE) 
        {
            $unterschied=1;
            $errortext.=" Unterschied bei Tel (".$address->getTelephone().") vs. (".(string)$party_data->ADDRESS->PHONE.") gefunden, ";
        }
        if($address->getFax()!=(string)$party_data->ADDRESS->FAX) 
        {
            $unterschied=1;
            $errortext.=" Unterschied bei Fax (".$address->getFax().") vs. (".(string)$party_data->ADDRESS->FAX.") gefunden, ";
        }
        if($address->getVatId()!=(string)$party_data->ADDRESS->VAT_ID) 
        {
            $unterschied=1;
            $errortext.=" Unterschied bei SteuerNr (".$address->getVatId().") vs. (".(string)$party_data->ADDRESS->VAT_ID.") gefunden, ";
        }
        
        //wenn unterschied vorhanden aktualisiere daten und setze adresse
        if($unterschied==1)
        {
            $address->setStoreId(Heliumv_Synchronization_Helper_Master::getStoreId());
            $address->setAttributeSetId(Heliumv_Synchronization_Helper_Master::getDefaultAttributeSetId('customer/address'));
            
            //wenn kundenid übergeben wurde weise diese adresse dieser kundenID zu
            if($customer_id!=null && $external_aid!=null)
            {
                $address->setParentId($customer_id);
                $address->setExternalAid($external_aid);
            }    
            
            $address->setPrefix((string)$party_data->ADDRESS->CONTACT_DETAILS->TITLE);  
            $address->setFirstname((string)$party_data->ADDRESS->CONTACT_DETAILS->FIRST_NAME);  
            $address->setLastname((string)$party_data->ADDRESS->CONTACT_DETAILS->CONTACT_NAME);
            $address->setCompany((string)$party_data->ADDRESS->NAME);
            $address->setStreet((string)$party_data->ADDRESS->STREET);
            $address->setPostcode((string)$party_data->ADDRESS->ZIP);
            $address->setCity((string)$party_data->ADDRESS->CITY); 
            $address->setCountryId((string)$party_data->ADDRESS->COUNTRY_CODED);
            $address->setTelephone((string)$party_data->ADDRESS->PHONE);
            $address->setFax((string)$party_data->ADDRESS->FAX);
            $address->setVatId((string)$party_data->ADDRESS->VAT_ID); 
            
            //hier gibt es möglicherweise noch fehler mit sql weil die spalte nachder sortiert werden soll nicht eindeutig ist, hmmm magento bug, workaround benötigt?
            try
            {
                $address->save();  
                mage::log("Bei AdressID:".$magento_aid." - (".(string)$party_data->ADDRESS->NAME.") folgende Unterschiede festgestellt (".$errortext.") und Adresse neu gespeichert",zend_log::INFO,self::$logfile_name);
                $return = $magento_aid;
            }
            catch(Exception $e)
            {
                mage::log("Exception beim Speichern von AdressID:".$magento_aid." - ".$e->getMessage(),zend_log::ERR,self::$logfile_name); 
                $return = null;     
            }
        }
        else
        {
            if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                mage::log("Bei AdressID:".$magento_aid." - (".(string)$party_data->ADDRESS->NAME.") wurden keine Unterschiede festgestellt und Adresse nicht aktualisiert.",zend_log::INFO,self::$logfile_name);
        }
        return $return;
    }

    
    /**
    * Konvertiert eine externe HV Kunden-Id in eine Magento Customer ID, versuche zugriff zuerst via object, dann via sql
    * @param mixed $external_cid - Kundenid aus HV
    * @return wird keine gefunden kommt null zurück, sonnst die customer id aus magento
    */
    public function convertExternalCidToCustomerId($external_cid)
    {
        //$store_id=Mage::app()->getStore()->getId();
    
        //beispiel um product_id by attribut zu finden von http://fishpig.co.uk/blog/load-a-category-or-product-by-an-attribute.html  
        // Instantiate a product collection object
        $customer = Mage::getResourceModel('customer');
        //setze den jeweiligen shop
        $customer->setStoreId(Heliumv_Synchronization_Helper_Master::getStoreId());
        // Select which fields to load into the product, * will load all fields but it is possible to pass an array of
        $customer->addAttributeToSelect('external_cid');
        // Add Name filter
        $customer->addAttributeToFilter('external_cid', $external_cid);
        // Limit the collection to 1 result
        $customer->setCurPage(1)->setPageSize(1);
        // Load the collection
        $customer->load();
        
        if ($customer->getFirstItem()->getEntityId())
        {
            //sollte es die daten mehrfach geben wird immer der erste artikel zurückgegeben
            $customer_id=$customer->getFirstItem()->getEntityId();
        }
        else 
        {
            //prüfe zusätzlich ob auch direkt via sql nichts gefunden werden kann
            $resource = Mage::getSingleton('core/resource');
            $sql="SELECT a.entity_id FROM ".$resource->getTableName("customer_entity_varchar")." a, ".$resource->getTableName("eav_attribute")." b where b.attribute_code='external_cid' and a.entity_type_id=b.entity_type_id and a.attribute_id=b.attribute_id and a.value='".$external_cid."'";
            $db_read = $resource->getConnection('core_read');
            $sqlresult = $db_read->fetchRow($sql);
            if(isset($sqlresult["entity_id"]) && $sqlresult["entity_id"]!="")
                $customer_id=$sqlresult["entity_id"];
            else
                $customer_id=null;
        }
        return $customer_id;
    }
    
    /**
    * Konvertiert eine externe HV Kunden Adress-Id in eine Magento AdressID, versuche zugriff via sql (objectzugriff gescheitert)
    * @param mixed $external_aid - Adressid aus HV
    * @return wird keine gefunden kommt null zurück, sonnst die adress id aus magento
    */
    public function convertExternalAidToAdressId($external_aid)
    {
        
        //prüfe via sql ob address entity gefunden werden kann
        $resource = Mage::getSingleton('core/resource');
        $sql="SELECT a.entity_id FROM ".$resource->getTableName("customer_address_entity_varchar")." a, ".$resource->getTableName("eav_attribute")." b where b.attribute_code='external_aid' and a.entity_type_id=b.entity_type_id and a.attribute_id=b.attribute_id and a.value='".$external_aid."'";
        $db_read = $resource->getConnection('core_read');
        $sqlresult = $db_read->fetchRow($sql);
        if(isset($sqlresult["entity_id"]) && $sqlresult["entity_id"]!="")
        {
            $adress_id=$sqlresult["entity_id"];
        }
        else
        {
            $adress_id=null;
        }
            
        return $adress_id;
    }
    
    /**
    * Konvertiert eine Magento Customer ID in eine externe HV Kunden-Id (wird für order benötigt)
    * versuche zugriff zuerst via object, dann via sql
    * @param mixed $magento_cid - Magento Customer ID
    * @return wird keine gefunden kommt null zurück, sonnst die HV Kunden-Id
    */
    public function convertInternalCidToExternalCustomerId($magento_cid)
    {
        //$store_id=Mage::app()->getStore()->getId();
        $customer = Mage::getModel('customer/customer');
        $customer->setStoreId(Heliumv_Synchronization_Helper_Master::getStoreId());
        $customer->load($magento_cid);
        
        if ($customer->getExternalCid() && $customer->getExternalCid()!="")
        {
            $customer_id=$customer->getExternalCid();
        }
        else 
        {
            //prüfe zusätzlich ob auch direkt via sql nichts gefunden werden kann
            $resource = Mage::getSingleton('core/resource');
            $sql="SELECT a.value FROM ".$resource->getTableName("customer_entity_varchar")." a, ".$resource->getTableName("eav_attribute")." b where b.attribute_code='external_cid' and a.entity_type_id=b.entity_type_id and a.attribute_id=b.attribute_id and a.entity_id='".$magento_cid."'";
            $db_read = $resource->getConnection('core_read');
            $sqlresult = $db_read->fetchRow($sql);
            if(isset($sqlresult["value"]) && $sqlresult["value"]!="")
                $customer_id=$sqlresult["value"];
            else
                $customer_id=null;
        }
        return $customer_id;
    }
    
    /**
    * Konvertiert eine Magento Adress ID in eine externe HV Adress-Id (wird für order benötigt)
    * versuche zugriff zuerst via object, dann via sql
    * @param mixed $magento_aid - Magento Adress ID
    * @return wird keine gefunden kommt null zurück, sonnst die HV Adress-Id
    */
    public function convertInternalAidToExternalAdressId($magento_aid)
    {
        //$store_id=Mage::app()->getStore()->getId();
        $adress = Mage::getModel('customer/address');
        $adress->setStoreId(Heliumv_Synchronization_Helper_Master::getStoreId());
        $adress->load($magento_aid);
        
        if ($adress->getExternalAid() && $adress->getExternalAid()!="")
        {
            $adress_id=$adress->getExternalAid();
        }
        else 
        {
            //prüfe zusätzlich ob auch direkt via sql nichts gefunden werden kann
            $resource = Mage::getSingleton('core/resource');
            $sql="SELECT a.value FROM ".$resource->getTableName("customer_address_entity_varchar")." a, ".$resource->getTableName("eav_attribute")." b where b.attribute_code='external_aid' and a.entity_type_id=b.entity_type_id and a.attribute_id=b.attribute_id and a.entity_id='".$magento_aid."'";
            $db_read = $resource->getConnection('core_read');
            $sqlresult = $db_read->fetchRow($sql);
            if(isset($sqlresult["value"]) && $sqlresult["value"]!="")
                $adress_id=$sqlresult["value"];
            else
                $adress_id=null;
        }
        return $adress_id;
    }
    
    
    /**
    * Ermittelt CustomerGroup oder erstellt Sie wenn nicht gefunden fix mit dem Namensformat HeliumPreislistenname(PreislistenID)
    * @param mixed $external_id entspricht der externen Gruppennummer 
    * @return liefert die Magnto ID der CustomerGroup zurück
    */
    public function checkCustomerGroup($external_id)
    {
        //variable init
        $group_id=null;
        $resource = Mage::getSingleton('core/resource');
        $return_group_name=null;
        $mapping_id_found=null;
        
        //versuche external_preislisten_id zu gruppenid aufgrund der magento config zu ermitteln
        $mapping_groupid_preislistid=Mage::getStoreConfig('synchronization/hidden_config/mapping_groupid_preislistid');
        if(isset($mapping_groupid_preislistid) && $mapping_groupid_preislistid!="")
        {
            $mapping_groupid_preislistid_array=unserialize($mapping_groupid_preislistid);
            if(isset($mapping_groupid_preislistid_array[(int)$external_id]) && $mapping_groupid_preislistid_array[(int)$external_id]!="")
            {
                $mapping_id_found=$mapping_groupid_preislistid_array[(int)$external_id];
                
                //ermittle name zu ID
                $sql="SELECT customer_group_code FROM ".$resource->getTableName("customer_group")." where customer_group_id='".$mapping_id_found."'";
                $db_read = $resource->getConnection('core_read');
                $sqlresult = $db_read->fetchRow($sql);
                if(isset($sqlresult["customer_group_code"]) && $sqlresult["customer_group_code"]!="")
                {
                    $return_group_name=$sqlresult["customer_group_code"];
                }
                
                //wenn kundengruppe nach wie vor Namen Error hat -> Schreibe Degug lok
                if(substr($return_group_name,0,5)=="Error" && Heliumv_Synchronization_Helper_Master::getDebugFlag()==1)
                    mage::log("Kundengruppe mit Preisliste/ID ".$mapping_id_found."/".$external_id." hat den Namen Error, bitte manuell im Backend umbenennen.",zend_log::WARN,self::$logfile_name);
                
                //wenn name nicht null -> liefere ID zurück
                if($return_group_name!=null)
                    return $mapping_id_found;
            }
        }
        
        //ermittle gruppenname aus helium
        $group_name_hv=$this->getCustomerGroupName($external_id);
        if($group_name_hv==null || !isset($group_name_hv->return->pricelist->cnr))
        {
            mage::log("Problem bei Ermittlung von Kundengruppen/Preislistenname mit ID:".$external_id." aufgetreten, nimm Name Error als Fallback",zend_log::ERR,self::$logfile_name);
            $group_name_hv="Error"; 
        }
        else
        {
            $group_name_hv=$group_name_hv->return->pricelist->cnr;    
        }
        
        //generiere neuen gruppen namen, darf maximal 32 zeichen haben
        $extention="(".$external_id.")";
        $new_group_name=substr(substr($group_name_hv,0,(32-strlen($extention))).$extention,0,32);
        
        //suche ob gruppe mit diesem namen vorhanden ist
        $sql="SELECT customer_group_id, customer_group_code FROM ".$resource->getTableName("customer_group")." where customer_group_code='".$new_group_name."'";
        $db_read = $resource->getConnection('core_read');
        $sqlresult = $db_read->fetchRow($sql);
        if(isset($sqlresult["customer_group_id"]) && $sqlresult["customer_group_id"]!="")
        {
            $group_id=$sqlresult["customer_group_id"];
        }
        else 
        {
            $model_new  = Mage::getModel('customer/group');
            $model_new->setData("customer_group_code",$new_group_name);
            $model_new->setData("tax_class_id",3);  // hardcodiert 3 bei initialisierung durch dynamischen wert ersetzen
            $model_new->save();
            $group_id=$model_new->getId();
            mage::log("Neue Customer Group mit Namen:".$new_group_name." und ID:".$group_id." angelegt.",zend_log::INFO,self::$logfile_name);
            
        }
        
        //gefundene oder neu angelegte gruppe auf jeden fall in zentralem mapping hinzufügen und speichern
        $mapping_groupid_preislistid=Mage::getStoreConfig('synchronization/hidden_config/mapping_groupid_preislistid');
        if(isset($mapping_groupid_preislistid) && $mapping_groupid_preislistid!="")
        {
            $mapping_groupid_preislistid_array=unserialize($mapping_groupid_preislistid);
            $mapping_groupid_preislistid_array[$external_id]=(int)$group_id;
        }
        else
        {
            $mapping_groupid_preislistid_array[$external_id]=(int)$group_id;    
        }
        Mage::getConfig()->saveConfig('synchronization/hidden_config/mapping_groupid_preislistid', serialize($mapping_groupid_preislistid_array) );
        Mage::getConfig()->reinit();
        Mage::app()->reinitStores();
        Mage::log("checkCustomerGroup - fuege Mapping von MagentoCustomergroup (".$group_id.") auf HeliumPreislistenId (".$external_id.") in Magento Config hinzu.",zend_log::INFO,self::$logfile_name);   
        
        return (int)$group_id;
    }
    
    
    
    /**
    * Test Customer Save via IMport
    * 
    */
    public function test_fast_save_customer()
    {
        $data = array();
        
        $data[]=
        array(
            "email" => "testxxx@testiurfieru".rand(1,99).".com" , //mandatory  - unique identifier
            "_website" => Heliumv_Synchronization_Helper_Master::getWebsiteCode() , //mandatory
            "_store" => "" ,
            "confirmation" => "" ,
            "created_at" => "" ,
            "created_in" => "" ,
            "disable_auto_group_change" => "" ,
            "dob" => "" ,
            "external_cid" => "3434" ,   //hv customer id
            "firstname" => "Vorname" ,
            "gender" => "Male" ,  //Male / Female
            "group_id" => "1" , //mandatory
            "lastname" => "Nachname" ,
            "middlename" => "" ,
            "password_hash" => "" ,
            "prefix" => "" ,
            "rp_token" => "" ,
            "rp_token_created_at" => "" ,
            "store_id" => "" ,
            "suffix" => "" ,
            "taxvat" => "" ,
            "website_id" => Heliumv_Synchronization_Helper_Master::getWebsiteId() , 
            "password" => "" ,
            "_address_city" => "Salzburg" ,
            "_address_company" => "HeliumV" ,
            "_address_country_id" => "AT" ,
            "_address_external_aid" => "2341234" , //hv adress id
            "_address_fax" => "076546547654-23" ,
            "_address_firstname" => "Adress_Firstname" ,
            "_address_lastname" => "Adress_Lastname" ,
            "_address_middlename" => "" ,
            "_address_postcode" => "5020" ,
            "_address_prefix" => "" ,
            "_address_region" => "" ,
            "_address_street" => "Street 22" ,
            "_address_suffix" => "" ,
            "_address_telephone" => "08674873487" ,
            "_address_vat_id" => "ATU87458745" ,
            "_address_default_billing_" => "" ,
            "_address_default_shipping_" => ""
            );
        for($x=0;$x<5;$x++)
        {    
            //beispiel für 1-n zusatzadresse zu kunden oben anfügen
            $data[]=        array(
                "email" => "" , 
                "_website" => "" , 
                "_store" => "" ,
                "confirmation" => "" ,
                "created_at" => "" ,
                "created_in" => "" ,
                "disable_auto_group_change" => "" ,
                "dob" => "" ,
                "external_cid" => "" ,   
                "firstname" => "" ,
                "gender" => "" ,  
                "group_id" => "" , 
                "lastname" => "" ,
                "middlename" => "" ,
                "password_hash" => "" ,
                "prefix" => "" ,
                "rp_token" => "" ,
                "rp_token_created_at" => "" ,
                "store_id" => "" ,
                "suffix" => "" ,
                "taxvat" => "" ,
                "website_id" => "" , 
                "password" => "" ,
                "_address_city" => "Koppl" ,
                "_address_company" => "" ,
                "_address_country_id" => "AT" ,
                "_address_external_aid" => "3453434".$x , //hv adress id
                "_address_fax" => "05554342-23" ,
                "_address_firstname" => "Adress_Koppl_Firstname".$x ,
                "_address_lastname" => "Adress_Koppl_Lastname".$x ,
                "_address_middlename" => "" ,
                "_address_postcode" => "5321" ,
                "_address_prefix" => "" ,
                "_address_region" => "" ,
                "_address_street" => "Street 22" ,
                "_address_suffix" => "" ,
                "_address_telephone" => "08674873487" ,
                "_address_vat_id" => "ATU87458745" ,
                "_address_default_billing_" => "" ,   //standardrechnungsadresse wenn auf 1
                "_address_default_shipping_" => ""    //standardlieferadresse wenn auf 1
                );
        }
        
        $time = microtime(true);
        mage::log("Start Import Kunde ",null,"test_import_Customer_v1.log");
        try {
            if(!file_exists(mage::getBaseDir("media").DIRECTORY_SEPARATOR."import"))
                mkdir(mage::getBaseDir("media").DIRECTORY_SEPARATOR."import");
            
            /** @var $import AvS_FastSimpleImport_Model_Import */
            $import = Mage::getModel('fastsimpleimport/import');
            $import
                ->setPartialIndexing(true)
                ->setBehavior(Mage_ImportExport_Model_Import::BEHAVIOR_APPEND)
                ->processCustomerImport($data);
            mage::log('Dauer: ' . round(microtime(true) - $time, 2) . 's',null,"test_import_Customer_v1.log");
            return "OK";
        } catch (Exception $e) {
            mage::log("Exception:".print_r($import->getErrorMessages(),1)." - ".$e->getMessage()." trace:\n".$e->getTraceAsString(),zend_log::ERR,"test_import_Customer_v1.log");
            return null;
        }
        
    }
    


    
    
    

}

?>