<?php
/**
* HELIUM V Synchronisation Modul for Magento 1.7.x
*
* Copyright (C) 2013 HELIUM V IT-Solutions GmbH
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU Affero General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* According to sec. 7 of the GNU Affero General Public License, version 3,
* the terms of the AGPL are supplemented with the following terms:
*
* "Helium V" and "Helium 5" are registered trademarks of HELIUM V IT-Solutions GmbH.
* The licensing of the Program under the AGPL does not imply a trademark license.
* Therefore any rights, title and interest in our trademarks remain entirely with us.
* If you want to propagate modified versions of the Program under the
* name "Helium V" or "Helium 5", you may only do so if you have a written permission
* by Helium V IT-Solutions GmbH (to acquire a permission please contact
* Helium V IT-Solutions at trademark@heliumv.com).
*      
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Affero General Public License for more details.
*
* You should have received a copy of the GNU Affero General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
* 
* @category Heliumv
* @package  Heliumv_Synchronization
* @author   Maximilian Hehenwarter <developers@heliumv.com>
* @license  http://www.gnu.org/licenses/  Affero General Public License (AGPL 3.0) 
*/

/**
* Hier werden alle Datenverarbeitungen und Helper für den Ordersync bzw. den Aufruf createOrder abgelegt
*/
class Heliumv_Synchronization_Model_Order extends Mage_Core_Model_Abstract
{
    
    /** variablendeklaration logfile für order, zugriff mit self::$logfile_name */
    private static $logfile_name="hv_ordersync.log";
    /** variablendeklaration der HV status für order sync */
    const STATUS_TRANSFER_START  = 'transfer_start';
    /** variablendeklaration der HV status für order sync */
    const STATUS_TRANSFER_COMPLETE  = 'transfer_complete';
    /** variablendeklaration der HV status für order sync */
    const STATUS_TRANSFER_ERROR  = 'transfer_error';
    /** variablendeklaration für status einer neuen order */
    const STATUS_NEW  = 'pending';
    
    /**
    * testfunction
    */
    public function test()
    {
        return "444";
    }
    
    
    /**
    * Hauptaufruf Ordersync, prüfe ob HV erreichbar, geht durch alle offenen Bestellungen und sendet diese, Errorhandling für hängende Bestellungen
    */
    public function doOrderSync()
    {        
        
        //schritt 1: wenn Helium nicht erreichbar abbruch
        $ping=Heliumv_Synchronization_Helper_Master::doPingHV();
        if($ping!=1)
            return null;
        
        mage::log("########## doOrderSync start #########",zend_log::INFO,self::$logfile_name);
            
        //schritt 2: Alle Bestellungen mit state "new" laden       
        $collection = Mage::getModel('sales/order')->getCollection()
            ->addAttributeToFilter('state', Mage_Sales_Model_Order::STATE_NEW)
            ->load();
            
        //für debug zwecke order immer laden unabhängig vom state
        //$collection = Mage::getModel('sales/order')->getCollection()->load();
           
        //schritte 3: gehe durch alle bestellungen 
        foreach ($collection as $order)
        {
            //debug - hardcodiert nimm immer orderid X wegen speziellem testfall
            //$order=Mage::getModel('sales/order')->load(4);
            
            try
            {
                //schreibe gesamtes order object in file wenn debug aktiv
                if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                    Heliumv_Synchronization_Helper_Master::debugLog('order'.$order->getId().'.txt',$order,"no",gmdate("Ymd"));
                
                
                //schritt 4: Setze (Status = extern) und (state=magento intern) der bestellung transaktionssicher auf transfer_start
                //$return_start=$this->setOrderStatusAndState($order,"transfer_start",Mage_Sales_Model_Order::STATE_PROCESSING);
                $return_start=$this->setOrderStatusAndState($order,self::STATUS_TRANSFER_START,self::STATUS_NEW);
                if($return_start==null)
                {
                    Heliumv_Synchronization_Helper_Master::sendErrorToHV(401,"Fehler beim Ändern des Order-Satus von ".self::STATUS_NEW." auf ".self::STATUS_TRANSFER_START."",3,"order",$order->getId());
                    continue; //fehler beim setzen des status -> gehe zu nächster Bestellung
                }
                
                //schritt 5: erstelle OrderXML aus Order-objekten
                $xml=$this->createOrderXML($order);
                
                //schritt 6: übermittle order via soap call an HV
                $return_soap=$this->sendSoapCreateOrder($order,$xml);
                
                //schritt 7: return prüfen und verarbeiten
                if($return_soap==null)
                {
                    Heliumv_Synchronization_Helper_Master::sendErrorToHV(402,"Fehler beim Übertragen von Order ".$order->getId()." via SOAP",3,"order",$order->getId());
                    mage::log("doOrderSync - Abbruch von OrderID:".$order->getId()." verarbeitung da Soap Cal NOK",zend_log::ERR,self::$logfile_name);
                    continue; //fehler beim soap call -> gehe zu nächster Bestellung
                } 
                    
                //speichere AuftragsNummer in Order
                try
                {
                    $order->setExtOrderId($return_soap);
                    $order->save();
                }
                catch(Exception $e)
                {
                    mage::log("setExtOrderID - OrderID:".$order->getId()." - Exception:".$e->getMessage(),zend_log::ERR,self::$logfile_name);
                }  
                
                //schritt 8: setze status auf bestellung übertragen
                $return_end=$this->setOrderStatusAndState($order,self::STATUS_TRANSFER_COMPLETE,self::STATUS_TRANSFER_START);
                
                //debug - verarbeite nur maximal eine bestellung
                //break;
                
            }
            catch (Exception $e)
            {
                Heliumv_Synchronization_Helper_Master::sendErrorToHV(403,"Allgemeinen Fehler bei Verarbeitung von OrderID ".$order->getId()." aufgetreten, Exception:".$ex->getMessage(),3,"order",$order->getId());
                Mage::log("Allgemeinen Fehler bei Verarbeitung von OrderID ".$order->getId()." aufgetreten, Exception:".$ex->getMessage(),zend_log::ERR,self::$logfile_name);    
                Mage::logException($e);
            }
        }    
        
        //Errorhandling wenn bestellungen hängen inkls. email alarm
        $this->checkOrderRetry();
        
        //errormeldung an hv für alle bestellungen die in sonstigen stati hängen
        $this->checkAllOtherOrders();
        
        mage::log("########## doOrderSync finished #########",zend_log::INFO,self::$logfile_name);
        
    }
    /**
    * Prüfe bestehenden Order-Status und setzt den Orderstatus auf einen neuen Wert, speichert dann diesen und prüft danach ob er gesetzt wurde. Der order State wird automatisch vom order-Status abgeleitet und auch geprüft/gesetzt. Der Ablauf ist transaktionsgesichert.
    * @param Mage_Sales_Model_Order $order
    * @param orderstatus $status_new - neuer Status der gesetzt werden soll (wird auch an Kunden kommuniziert und kann textuell gepflegt werden)
    * @param orderstate $status_old - optional bestehender Status, wenn übergeben wird geprüft ob bestellung tatsächlich diesen status hat, wenn nicht wird status nicht geändert
    * @param mixed $comment - Kommentar zu Änderung wird gespeichert wenn string befüllt
    * @param mixed $visible_to_customer - Kommentar und Änderung wird dem Kunden angezeigt, ja=true, nein=null (default)
    * @return Liefert 1=true zurück wenn alles ok war und null wenn ein fehler aufgetreten ist
    */
    private function setOrderStatusAndState(Mage_Sales_Model_Order $order,$status_new,$status_old=null,$comment=null,$visible_to_customer=null)
    {
        //variablen initialisieren
        $return = null;
        $order_id=(int)$order->getId();
        if($order_id==null || $order_id==0)
        {
            mage::log("setOrderStatusAndState - OrderID ist ungültig bei Status auf ".$status_new." setzen, -> Abbruch Verarbeitung",zend_log::ERR,self::$logfile_name);
            return null;
        }
        
        //debug option set -> die order mut nummer X bekommt kein Statusupdate
        $dont_update_orderstatus=(int)Mage::getStoreConfig('synchronization/debug/dont_update_orderstatus');
        if($order_id==$dont_update_orderstatus)
        {
            mage::log("setOrderStatusAndState - OrderID:".$order_id." Status:".$status_new." nicht gesetzt da Debug-Flag \"Not Update Status on OrderNr:\" fuer diese OrderNummer aktiv",zend_log::WARN,self::$logfile_name);
            return true; //bewusst true zurüpckgebeben und keine weitere verarbeitung
        }
        
        //info zu order status vs state: http://ka.lpe.sh/2012/04/21/magento-difference-between-order-states-and-statuses/
        //order state verwendet magento intern zur verarbeitung, order status kann man extern setzen
        try
        {
            //setze lock bzw warte auf nächsten freien lock
            $lock = new Mage_Index_Model_Process();
            $lock->setId("change_order_status");
            $lock_wait=microtime(1);
            $lock->lockAndBlock();
            if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                mage::log("setOrderStatusAndState - Dauer getlock change_order_status: ".number_format(microtime(1)-$lock_wait,4),zend_log::DEBUG,self::$logfile_name);
            
            //alten Status/state aus order lesen
            $state_old_orderobj=$order->getState();
            $status_old_orderobj=$order->getStatus();
            
            //prüfe gegen status alt wenn übergeben
            if($status_old!=null)
            {
                //ermittle state zu status aus datenquelle
                $state_old=$this->convertOrderStatusToState($status_old);
                //ermittle ob unterschiede von status und state vorhanden
                if($state_old!=$state_old_orderobj || $status_old!=$status_old_orderobj)
                {
                    mage::log("setOrderStatusAndState - OrderID:".$order_id." hat falschen bestehenden Status -> Abbruch. Order Status/State (".$status_old_orderobj."/".$state_old_orderobj."), Uebergeben Status/State (".$status_old."/".$state_old.")",zend_log::ERR,self::$logfile_name);
                    return null;         
                }   
            }
            
            //ermittle state zu status aus datenquelle
            $state_new=$this->convertOrderStatusToState($status_new);        
            
            //neuen status/state setzen
            $order->setStatus($status_new);
            $order->setState($state_new);
            
            //erstelle order status history object
            //input von http://stackoverflow.com/questions/6219620/add-username-to-order-comment-history
            $status_history_obj= Mage::getModel('sales/order_status_history');
            $status_history_obj->setStatus($status_new);
            $status_history_obj->setIsCustomerNotified(0); //löse kein email aus
            $status_history_obj->setIsVisibleOnFront(0); //nicht dem endkunden anzeigen
            $status_history_obj->setEntityName('order');
            if($comment!=null && $comment!="")
                $status_history_obj->setComment($comment);
            if($visible_to_customer!=null && $visible_to_customer!=false && $visible_to_customer!="")
                $status_history_obj->setIsVisibleOnFront(1);  //dem endkunden anzeigen
            
            //order status history der order hinzufügen    
            $order->addStatusHistory($status_history_obj);

            //speichere order (stati + history)                
            $order->save();  
            
            //prüfe ob neuer status/state in object gesetzt ist
            $objnewstate=$order->getState();
            $objnewstatus=$order->getStatus();
            //prüfe ob neuer status in db gesetzt ist
            $resource = Mage::getSingleton('core/resource');
            $sql="SELECT status,state FROM ".$resource->getTableName("sales_flat_order")." where entity_id='".$order->getId()."'"; 
            $db_read = $resource->getConnection('core_read');
            $sqlresult = $db_read->fetchRow($sql);
            $sqlnewstate = $sqlresult ['state']; 
            $sqlnewstatus = $sqlresult ['status'];
            
            //vergleiche neue werte mit sql und objectergebniss
            if($status_new==$objnewstatus && $status_new==$sqlnewstatus && $state_new==$objnewstate && $state_new==$sqlnewstate)
            {
                $return = true;
                if(Heliumv_Synchronization_Helper_Master::getPositiveLogFlag())
                    mage::log("setOrderStatusAndState - State/Status von OrderID".$order_id." von ".$state_old_orderobj."/".$status_old_orderobj." auf ".$state_new."/".$status_new." geaendert.",zend_log::INFO,self::$logfile_name);
            }
            else
            {
                $return=null;
                mage::log("setOrderStatusAndState - Fehler beim Setzen von State/Status von OrderID".$order_id." von ".$state_old_orderobj."/".$status_old_orderobj." auf ".$state_new."/".$status_new." geaendert.",zend_log::ERR,self::$logfile_name);
            }

            //beende lock
            $lock->unlock(); 
            
        }
        catch(Exception $e)
        {
            //beende lock
            $lock->unlock(); 
            mage::log("setOrderStatusAndState - Exception:".$e->getMessage(),zend_log::ERR,self::$logfile_name);
            $return=null;
        }
        
        return $return;
    }
    
    /**
    * Ermittelt den Order State zu einem Order Status
    * @param mixed $status
    * @return mixed $state - retuniert order state zu status, wenn nicht gefunden null
    */
    private function convertOrderStatusToState($status)
    {
        $return=null;
        
        //state zu status ermitteln
        $resource = Mage::getSingleton('core/resource');
        $sql="SELECT state FROM ".$resource->getTableName("sales_order_status_state")." where status='".$status."'"; 
        $db_read = $resource->getConnection('core_read');
        $sqlresult = $db_read->fetchRow($sql);
        $state_from_sql = $sqlresult ['state']; 
            
        //$state_from_obj=Mage::getModel('sales/order_status_state')->load($status,'status')->getData('state');
        if($state_from_sql!=null || $state_from_sql!="")
        {
            $return=$state_from_sql;        
        }
        else
        {
            Mage::log("convertOrderStatusToState - Fehler beim Ermitteln von State zu Status ".$status."",zend_log::ERR,self::$logfile_name);
            $return=null;
        }
        return $return;
    }
    

    /**
     * Übermittelt Magento Order an Helium
     *
     * @param Mage_Sales_Model_Order $order
     * @return Webkochshop_OrderExport_Model_Observer
     */
    /*private function OLD_sendSingleOrder(Mage_Sales_Model_Order $order)
    {

        $attributes = $order->getData();
        $attributes = array_keys($attributes);
        
        $options = array('trace' => 1);
        Heliumv_Synchronization_AjaxController::$logonData = $this->initConfig();

        $server_url=Heliumv_Synchronization_Helper_Master::getWebserviceUrlOrder();
        Heliumv_Synchronization_AjaxController::$client = new SoapClient($server_url,$options);

        
        //create order xml
        
        //SimpleXMLElement::addAttribute()
        
        
        // XXXXXXXXX order - weiter bei order format .... daweil noch beispiel für daten empfangen, nicht senden
        try {
            // get all items from the heliumv instance 
            $result = Heliumv_Synchronization_AjaxController::$client->createOrder(array(
                'arg0' => Heliumv_Synchronization_AjaxController::$logonData)
            );

            // check response 
            if ($result->return->rc == 0) {
                // loop through the result set 
               
                
            } else {
                Mage::log(Heliumv_Synchronization_AjaxController::$client->__getLastRequest(),null,self::$logfile_name);
                $response = array('error' => true, 'message' => $result);
            }
        } catch (SoapFault $exc) {
            $response = array('error' => true, 'message' => $exc->getMessage());
        }


         //avoid HEADERS ALREADY SENT error; http://stackoverflow.com/questions/4441526/best-way-to-output-ajax-data-from-a-magento-admin-extension

        if (is_array($response)) {
            $response = Mage::helper('core')->jsonEncode($response);
            $this->getResponse()->setBody($response);
        }
        
    }*/
    
    /**
    * Sendet einzelne Ordner via Soap Call createOrder an ERP System, benötigt NuSoap.php als Libary installiert
    * @param Mage_Sales_Model_Order $order Order-Object
    * @param mixed $xml XML welches aus Order Object mittels Funktion createOrderXML erstellt wurde
    * @return Wenn OK liefert OrderNummer zurück, sonnst null
    */
    /*private function old_sendSoapCreateOrder(Mage_Sales_Model_Order $order,$xml)
    {
        $order_id=$order->getId();
        $file=Mage::getBaseDir('lib') . DIRECTORY_SEPARATOR.'nusoap-0.9.5'.DIRECTORY_SEPARATOR.'nusoap.php';
        include_once($file); //wird ab modul Heliumv_Synchronization-0.0.2 automatisch mit ausgeliefert
        $return=null;
        
        //$server_url="http://192.168.8.244:8080/lpserver-ejb/AuftragFacBeanRest?wsdl";
        $server_url=Heliumv_Synchronization_Helper_Master::getWebserviceUrlOrder();
        
        $nusoap = new nusoap_client($server_url,false);
        //$nusoap->setHeaders("<arg0><isoCountry>at</isoCountry><isoLanguage>de</isoLanguage><password>webshop1</password><user>webshop1</user></arg0>");
        $nusoaperr = $nusoap->getError();
        if ($nusoaperr) {
            //echo '<h2>Constructor error</h2><pre>' . $nusoaperr . '</pre>';
            mage::log("CreateOrder:".$order_id." - Nusoap Constructor error:".$nusoaperr,Zend_Log::ERR,self::$logfile_name);
            $return=null;
        }
        // Doc/lit parameters get wrapped
        //$nusoapparam = array('xmlOpenTransOrder' => "<![CDATA[".$xml->asXML()."]]>");
        $nusoapnamespace="http://ejbfac.auftrag.server.lp.com/";
        //$nusoapheaders="<ejb:arg0 xmlns:ejb=\"http://ejbfac.auftrag.server.lp.com/\"><isoCountry>at</isoCountry><isoLanguage>de</isoLanguage><password>webshop1</password><user>webshop1</user></ejb:arg0>";
        $nusoapheaders="<ejb:arg0 xmlns:ejb=\"http://ejbfac.auftrag.server.lp.com/\"><isoCountry>".Mage::getStoreConfig('synchronization/heliumvconnect/country')."</isoCountry><isoLanguage>".Mage::getStoreConfig('synchronization/heliumvconnect/language')."</isoLanguage><password>".Mage::getStoreConfig('synchronization/heliumvconnect/password')."</password><user>".Mage::getStoreConfig('synchronization/heliumvconnect/user')."</user><token>".Mage::getStoreConfig('synchronization/heliumvconnect/token')."</token><shopName>".Mage::getStoreConfig('synchronization/heliumvconnect/shop_name')."</shopName></ejb:arg0>";
        
        $nusoapresult = $nusoap->call('createOrder', array('xmlOpenTransOrder' => "<![CDATA[".$xml->ORDER->asXML()."]]>"), $nusoapnamespace, '',$nusoapheaders, true);
        // Check for a fault
        if ($nusoap->fault) {
            //echo '<h2>Fault</h2><pre>';
            //print_r($nusoapresult);
            //echo '</pre>';
            mage::log("CreateOrder:".$order_id." - NusoapCall Fault:".print_r($nusoapresult,true),Zend_Log::ERR,self::$logfile_name);
            $return=null;
        } else {
            // Check for errors
            $nusoaperr = $nusoap->getError();
            if ($nusoaperr) {
                // Display the error
                //echo '<h2>Error</h2><pre>' . $nusoaperr . '</pre>';
                mage::log("CreateOrder:".$order_id." - NusoapCall Error:".$nusoaperr,Zend_Log::ERR,self::$logfile_name);
                $return = null;
            } else {
                // Display the result
                //echo '<h2>Result</h2><pre>';
                //print_r($nusoapresult);
                //echo '</pre>';
                if(isset($nusoapresult["rc"]) && $nusoapresult["rc"]=="0" && isset($nusoapresult["orderNumber"]) && $nusoapresult["orderNumber"]!="")
                {
                    $return=$nusoapresult["orderNumber"]; 
                    mage::log("CreateOrderID:".$order_id." mit HVOrderNr:".$return." OK",zend_log::INFO,self::$logfile_name);   
                }
                elseif(isset($nusoapresult["rc"]) && $nusoapresult["rc"]=="10014" && isset($nusoapresult["description"]) && $nusoapresult["description"]!="") 
                {
                    //spezialfall, dass bestellung in helium schon fertig angelegt wurde (zb verarbeitungszeigt helium lönger als soap timeout)
                    $return=$nusoapresult["description"]; 
                    mage::log("CreateOrderID:".$order_id." ist bereits in Helium mit HVOrderNr:".$return." vorhanden gewesen (Erkannt durch RC 10014), uebernimm diese Nummer als externeNummer in Magento",zend_log::WARN,self::$logfile_name);           
                }            
                else
                {
                    if(!isset($nusoapresult["rc"]))
                        $nusoapresult["rc"]="not_set";
                    if(!isset($nusoapresult["orderNumber"]))
                        $nusoapresult["orderNumber"]="not_set";
                    
                    mage::log("CreateOrder:".$order_id." - Problem mit RC:".$nusoapresult["rc"]." oder HVOrderNr:".$nusoapresult["orderNumber"],Zend_Log::ERR,self::$logfile_name);
                        
                    $return = null;
                }
                    
                
                //mage::log("CreateOrder:".$order_id." - NusoapCall OK:".print_r($nusoapresult,true));
                
            }
        }
        
        //echo '<h2>Request</h2><pre>' . htmlspecialchars(str_replace("><",">\n<",$nusoap->request), ENT_QUOTES) . '</pre>';
        //echo '<h2>Response</h2><pre>' . htmlspecialchars(str_replace("><",">\n<",$nusoap->response), ENT_QUOTES) . '</pre>';
        //echo '<h2>Debug</h2><pre>' . htmlspecialchars($nusoap->debug_str, ENT_QUOTES) . '</pre>';      
        
        //schreibe gesamten call in file wenn debug aktiv
        if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
        {
            Heliumv_Synchronization_Helper_Master::debugLog('order'.$order->getId().'_request.txt',str_replace("><",">\n<",$nusoap->request),"no",gmdate("Ymd"));  
            Heliumv_Synchronization_Helper_Master::debugLog('order'.$order->getId().'_response.txt',str_replace("><",">\n<",$nusoap->response),"no",gmdate("Ymd"));  
            Heliumv_Synchronization_Helper_Master::debugLog('order'.$order->getId().'_soapdebug.txt',$nusoap->debug_str,"no",gmdate("Ymd"));  
        }

        return $return;
        
    }
    */
    
    /**
    * Sendet einzelne Ordner via Soap Call createOrder an ERP System, benötigt NuSoap.php als Libary installiert
    * @param Mage_Sales_Model_Order $order Order-Object
    * @param mixed $xml XML welches aus Order Object mittels Funktion createOrderXML erstellt wurde
    * @return Wenn OK liefert OrderNummer zurück, sonnst null
    */
    private function sendSoapCreateOrder(Mage_Sales_Model_Order $order,$xml)
    {
        $options = array('trace' => 1);
        $login = Heliumv_Synchronization_Helper_Master::getWebserviceHeaderData();
        $server_url=Heliumv_Synchronization_Helper_Master::getWebserviceUrlOrder();
        $soap_timeout_request=Mage::getStoreConfig('synchronization/heliumvconnect/soap_timeout_request');
        $soap_timeout_response=Mage::getStoreConfig('synchronization/heliumvconnect/soap_timeout_response');
        $function_name=__FUNCTION__;
        $return=null;
        $order_id=$order->getId();
        $nusoapnamespace="http://ejbfac.auftrag.server.lp.com/";
        //nur fuer order call benötigt
        $nusoapheaders="<ejb:arg0 xmlns:ejb=\"http://ejbfac.auftrag.server.lp.com/\"><isoCountry>".Mage::getStoreConfig('synchronization/heliumvconnect/country')."</isoCountry><isoLanguage>".Mage::getStoreConfig('synchronization/heliumvconnect/language')."</isoLanguage><password>".Mage::getStoreConfig('synchronization/heliumvconnect/password')."</password><user>".Mage::getStoreConfig('synchronization/heliumvconnect/user')."</user><token>".Mage::getStoreConfig('synchronization/heliumvconnect/token')."</token><shopName>".Mage::getStoreConfig('synchronization/heliumvconnect/shop_name')."</shopName></ejb:arg0>";

        try
        {    
            //nusoap soapclient einbinden
            $file=Mage::getBaseDir('lib') . DIRECTORY_SEPARATOR.'nusoap-0.9.5'.DIRECTORY_SEPARATOR.'nusoap.php';
            include_once($file); //wird ab modul Heliumv_Synchronization-0.0.2 automatisch mit ausgeliefert
            
            //verbindung muss innerhalb von timeout stehen
            $nusoap = new nusoap_client($server_url,false,false, false, false, false, $soap_timeout_request, $soap_timeout_response);
            
            //change encoding settings -> soap client liefert bzw sendet utf8 -> bewirkt encoding="utf-8" in xml
            $nusoap->soap_defencoding = 'UTF-8';
            $nusoap->decode_utf8 = false;
            
            //fehler aufgetreten
            $nusoaperr = $nusoap->getError();
            if ($nusoaperr) {
                mage::log($function_name." Error - Nusoap Constructor error:".$nusoaperr,Zend_Log::ERR,self::$logfile_name);
                $return=null;
            }
            
            //get artikel details from the heliumv instance
            $start_time=microtime(1);
            $nusoapresult = $nusoap->call('createOrder', array('xmlOpenTransOrder' => "<![CDATA[".$xml->ORDER->asXML()."]]>"), $nusoapnamespace, '',$nusoapheaders, true);
            
            //prüfe auf fault
            if ($nusoap->fault) 
            {
                mage::log($function_name." Fault - NusoapCall Fault, Response:".print_r($nusoapresult,true),Zend_Log::ERR,self::$logfile_name);
                $return=null;
            } 
            else 
            {
                //prüfe auf fehler
                $nusoaperr = $nusoap->getError();
                if ($nusoaperr)
                {
                    //prüfe ob Timeout gekommen ist: server nicht erreichbar
                    if(substr($nusoaperr,0,53)=="HTTP Error: Couldn't open socket connection to server")
                    {
                        Heliumv_Synchronization_Helper_Master::createSoapTimeoutLog($function_name."_".$order_id,"Server nicht erreicht");
                        mage::log($function_name." Error - NusoapCall Error, Server nicht erreicht:".trim($nusoaperr),Zend_Log::ERR,self::$logfile_name);         
                    }
                    //prüfe ob Timeout gekommen ist:  erreichbar aber timeout
                    if(substr($nusoaperr,0,45)=="HTTP Error: socket read of headers timed out")
                    {
                        Heliumv_Synchronization_Helper_Master::createSoapTimeoutLog($function_name."_".$order_id,"Server erreicht aber Request kommt nicht zurueck");
                        mage::log($function_name." Error - NusoapCall Error, Server erreicht aber Request kommt nicht zurueck:".trim($nusoaperr),Zend_Log::ERR,self::$logfile_name);         
                    }
                    
                    $return=null;
                } 
                else
                {
                    //ermittle dauer rundtriptime für Call
                    $rtt=(microtime(1)-$start_time);  
                    
                    
                    if(isset($nusoapresult["rc"]) && $nusoapresult["rc"]=="0" && isset($nusoapresult["orderNumber"]) && $nusoapresult["orderNumber"]!="")
                    {
                        $return=$nusoapresult["orderNumber"]; 
                        
                        if(Heliumv_Synchronization_Helper_Master::getPositiveLogFlag())
                            mage::log($function_name." CreateOrderID:".$order_id." mit HVOrderNr:".$return." OK",zend_log::INFO,self::$logfile_name);   
                    }
                    elseif(isset($nusoapresult["rc"]) && $nusoapresult["rc"]=="10014" && isset($nusoapresult["description"]) && $nusoapresult["description"]!="") 
                    {
                        //spezialfall, dass bestellung in helium schon fertig angelegt wurde (zb verarbeitungszeigt helium lönger als soap timeout)
                        $return=$nusoapresult["description"]; 
                        mage::log($function_name." CreateOrderID:".$order_id." ist bereits in Helium mit HVOrderNr:".$return." vorhanden gewesen (Erkannt durch RC 10014), uebernimm diese Nummer als externeNummer in Magento",zend_log::WARN,self::$logfile_name);           
                    }            
                    else
                    {
                        if(!isset($nusoapresult["rc"]))
                            $nusoapresult["rc"]="not_set";
                        if(!isset($nusoapresult["orderNumber"]))
                            $nusoapresult["orderNumber"]="not_set";
                        
                        //spezielles Debug Feedback für spezielle Returncodes
                        if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                        {
                            if($nusoapresult["rc"]=="100")
                                mage::log($function_name." RC 100 deutet z.B. auf falschen oder fehlenden Token beim Webshopuser in HV hin.",zend_log::DEBUG,self::$logfile_name);
                        }
                        
                        mage::log($function_name." Error bei OrderID:".$order_id." - Problem mit RC:".$nusoapresult["rc"]." oder HVOrderNr:".$nusoapresult["orderNumber"],Zend_Log::ERR,self::$logfile_name);
                        $return = null;
                    }
                    
                }
            } //else kein fault
        }
        catch(Exception $e)
        {
            mage::log($function_name." - Exeption:".$e->getMessage(),zend_log::ERR,self::$logfile_name);       
            $return=null; 
        }    
        
        //schreibe gesamten call in file wenn debug aktiv
        if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
        {
            Heliumv_Synchronization_Helper_Master::debugLog($function_name.'_'.$order_id.'_'.date("His").'_request.txt',str_replace("><",">\n<",$nusoap->request),"no",gmdate("Ymd"));  
            Heliumv_Synchronization_Helper_Master::debugLog($function_name.'_'.$order_id.'_'.date("His").'_response.txt',str_replace("><",">\n<",$nusoap->response),"no",gmdate("Ymd"));  
            
            //debug nur im fehlerfall schreiben da große datenmenge
            if($return==null)
                Heliumv_Synchronization_Helper_Master::debugLog($function_name.'_'.$order_id.'_'.date("His").'_soapdebug.txt',$nusoap->debug_str,"no",gmdate("Ymd"));  
        }
        
        return $return ;
    }
    
    /**
    * Erstellt Order XML in UTF-8 für übermittlung an HV in Call createOrder in XML Tag xmlOpenTransOrder
    * @param Mage_Sales_Model_Order $obj
    * @return OrderXML für Tag xmlOpenTransOrder $xml
    */
    public function createOrderXML(Mage_Sales_Model_Order $obj)
    {
        //nicht mehr benötigt da direkt bei new SimpleXMLElement gesetzt
        //ermittle aktuelles in php eingestelltes internes encoding und convertiere ausgehende strings auf utf8
        //$enc2=iconv_get_encoding("internal_encoding");
        //$enc1='UTF-8'; //hv ss benötigt zwingend utf8
        //verwendungsbeispiel für strings unten
        //$xml_string = iconv($enc1,$enc2, $umlaut_string);  
        
        try
        {
            //durch das initialkisieren des XML Objects mit encoding='utf-8' werden die übergebenen strings automatisch auf utf8 convertiert
            //dadurch verwendung von iconv hinfällig, input von http://stackoverflow.com/questions/869650/getting-simplexmlelement-to-include-the-encoding-in-output
            $xml = new SimpleXMLElement("<?xml version='1.0' encoding='utf-8'?><xmlOpenTransOrder/>");
            
            //wird zur konvertierung der customerid und adressids benötigt
            $customer_modul=new Heliumv_Synchronization_Model_Customer();
            
            //lies adressobjecte zu order obj dazu
            $obj_shipping = Mage::getModel('sales/order_address')->load($obj->getShippingAddressId());
            $obj_billing = Mage::getModel('sales/order_address')->load($obj->getBillingAddressId());
            
            //wenn debug aktiv schreibe adressobjekte in files
            if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
            {
                Heliumv_Synchronization_Helper_Master::debugLog('order'.$obj->getId().'_rechaddr.txt',$obj_billing,"no",gmdate("Ymd"));
                Heliumv_Synchronization_Helper_Master::debugLog('order'.$obj->getId().'_liefaddr.txt',$obj_shipping,"no",gmdate("Ymd"));
            }

            //namespaces
            $ns_bmecat="http://www.bmecat.org/bmecat/2005";
            
            //header
            $order=$xml->addChild("ORDER");
            $order->addAttribute("type","standard");
            $order->addAttribute("version","2.1");
            $order->addAttribute("xmlns","http://www.opentrans.org/XMLSchema/2.1");
            $order->addAttribute("xmlns:xsi","http://www.w3.org/2001/XMLSchema-instance");
            $order->addAttribute("xsi:schemaLocation","http://www.opentrans.org/XMLSchema/2.1 opentrans_2_1.xsd");
            //$order->addAttribute("xmlns:bmecat","http://www.bmecat.org/bmecat/2005");
            $order->addAttribute("xmlns:xmime","http://www.w3.org/2005/05/xmlmime");
            $order->addAttribute("xmlns:xsig","http://www.w3.org/2000/09/xmldsig#");
            $order_header = $order->addChild("ORDER_HEADER");
            $order_header->addChild("CONTROL_INFO")->addChild("STOP_AUTOMATIC_PROCESSING","Vom Webshop erzeugt");
            $order_info = $order_header->addChild("ORDER_INFO");
            $order_info->addChild("ORDER_ID",preg_replace("/\\b0*/","",substr($obj->getIncrementId(),1)));   //orderid aus magento backend (nicht db-id) und führenden 1er und nullen wegnehmen
            $ts=strtotime($obj->getCreatedAtDate()); //Zeit ist bereits als UTC in DB gespeichert
            $order_info->addChild("ORDER_DATE",date("Y-m-d",$ts)."T".date("H:i:s",$ts));
            
            //shipping method in incoterms übertragen
            $transport=$order_info->addChild("bmecat:TRANSPORT",null,$ns_bmecat);
            $transport->addChild("INCOTERM",$obj->getShippingMethod());
            $transport->addChild("TRANSPORT_REMARK",$obj->getShippingDescription());

            //kundendaten und adressen
            $parties=$order_info->addChild("PARTIES");
            
            //kundendaten kopf 
            /*$party=$parties->addChild("PARTY");
            $party->addChild("bmecat:PARTY_ID",$obj->getCustomerId(),$ns_bmecat)->addAttribute("type","customer_specific"); //magento interne KundenID
            $party->addChild("bmecat:PARTY_ID",$customer_modul->convertInternalCidToExternalCustomerId($obj->getCustomerId()),$ns_bmecat)->addAttribute("type","heliumv"); //externe HV KundenID
            $party->addChild("bmecat:PARTY_ID",$obj->getCustomerGroupId(),$ns_bmecat)->addAttribute("type","customer_group"); //magento kunden gruppen id
            $party->addChild("PARTY_ROLE","buyer");  //geändert von customer auf buyer auf Wunsch von Gerold
            
            //kundendaten detail (alle pflichtfelder aus dem Onlineshop)
            $adress=$party->addChild("ADDRESS");
            $adress->addChild("bmecat:EMAIL",$obj->getCustomerEmail(),$ns_bmecat);
            $contact_details=$adress->addChild("CONTACT_DETAILS");
            $contact_details->addChild("bmecat:CONTACT_NAME",$obj->getCustomerLastname(),$ns_bmecat);
            $contact_details->addChild("bmecat:FIRST_NAME",$obj->getCustomerFirstname(),$ns_bmecat);
            $contact_details->addChild("bmecat:TITLE",$obj->getCustomerGender(),$ns_bmecat); */ 
            
            //berechne richtige adressids, tritt auf wenn man als neukunde sich anmeldet und zum ersten mal mit gleicher rech/lief-addr bestellt
            $shipping_addr_id=$obj_shipping->getCustomerAddressId();
            $billing_addr_id=$obj_billing->getCustomerAddressId();
            if((int)$shipping_addr_id==0 && (int)$billing_addr_id!=0)
            {
                // vergleiche adresse, wenn gleich dann übernimm id
                 if ($obj_shipping->format('html') == $obj_billing->format('html'))
                 {
                     $shipping_addr_id=$billing_addr_id;    
                 } 
            }
            if((int)$shipping_addr_id!=0 && (int)$billing_addr_id==0)
            {
                // vergleiche adresse, wenn gleich dann übernimm id
                if ($obj_shipping->format('html') == $obj_billing->format('html'))
                {
                    $billing_addr_id=$shipping_addr_id;    
                } 
            }
            //wenn beide adressIDs leer sind (beide adressen ohne speichern übergeben)
            if((int)$shipping_addr_id==0 && (int)$billing_addr_id==0)
            {
                //elemente auf leer setzen damit adresse in hv neu angelegt wird
                $shipping_addr_id="";
                $billing_addr_id="";
            }  
            
            //rechnungsadresse kopf
            $party=$parties->addChild("PARTY");
            $party->addChild("bmecat:PARTY_ID",$billing_addr_id,$ns_bmecat)->addAttribute("type","customer_specific"); //magento rechnungs adress ID
            $party->addChild("bmecat:PARTY_ID",$customer_modul->convertInternalAidToExternalAdressId($obj_billing->getCustomerAddressId()),$ns_bmecat)->addAttribute("type","heliumv"); //externe HV rechnungs AdressID
            $party->addChild("PARTY_ROLE","invoice_recipient");
            $party->addChild("PARTY_ROLE","buyer"); //statt kunde oben
            //rechnungsadressdetails 
            $adress=$party->addChild("ADDRESS");
            //wenn firmenname gesetzt, dann firmenname, sonnst leer
            $adress->addChild("bmecat:NAME",$obj_billing->getCompany(),$ns_bmecat);
            $contact_details=$adress->addChild("CONTACT_DETAILS");
            $contact_details->addChild("bmecat:CONTACT_ID",$billing_addr_id,$ns_bmecat); //magento adressid
            $contact_details->addChild("bmecat:CONTACT_NAME",$obj_billing->getLastname(),$ns_bmecat);
            $contact_details->addChild("bmecat:FIRST_NAME",$obj_billing->getFirstname(),$ns_bmecat);
            $contact_details->addChild("bmecat:TITLE",$obj_billing->getPrefix(),$ns_bmecat);
            $adress->addChild("bmecat:COUNTRY_CODED",$obj_billing->getCountryId(),$ns_bmecat);
            $adress->addChild("bmecat:PHONE",$obj_billing->getTelephone(),$ns_bmecat);
            $adress->addChild("bmecat:FAX",$obj_billing->getFax(),$ns_bmecat);
            $adress->addChild("bmecat:EMAIL",$obj->getCustomerEmail(),$ns_bmecat);
            $adress->addChild("bmecat:VAT_ID",$obj->getCustomerTaxvat(),$ns_bmecat);
            $street_array=$obj_billing->getStreet();
            //mehrzeilige adressen zusammenfügen
            if(isset($street_array[1]))
                $street_string=$street_array[0].", ".$street_array[1];    
            else
                $street_string=$street_array[0];        
            $adress->addChild("bmecat:STREET",$street_string,$ns_bmecat);
            $adress->addChild("bmecat:ZIP",$obj_billing->getPostcode(),$ns_bmecat);
            $adress->addChild("bmecat:CITY",$obj_billing->getCity(),$ns_bmecat);

            //lieferadresse kopf
            $party=$parties->addChild("PARTY");
            $party->addChild("bmecat:PARTY_ID",$shipping_addr_id,$ns_bmecat)->addAttribute("type","customer_specific"); //magento liefer adress ID
            $party->addChild("bmecat:PARTY_ID",$customer_modul->convertInternalAidToExternalAdressId($obj_shipping->getCustomerAddressId()),$ns_bmecat)->addAttribute("type","heliumv"); //externe HV liefer AdressID
            $party->addChild("PARTY_ROLE","delivery");
            //lieferadresse
            $adress=$party->addChild("ADDRESS");
            //wenn firmenname gesetzt, dann firmenname, sonnst leer
            $adress->addChild("bmecat:NAME",$obj_shipping->getCompany(),$ns_bmecat);
            $contact_details=$adress->addChild("CONTACT_DETAILS");
            $contact_details->addChild("bmecat:CONTACT_ID",$shipping_addr_id,$ns_bmecat); //magento adressid
            $contact_details->addChild("bmecat:CONTACT_NAME",$obj_shipping->getLastname(),$ns_bmecat);
            $contact_details->addChild("bmecat:FIRST_NAME",$obj_shipping->getFirstname(),$ns_bmecat);
            $contact_details->addChild("bmecat:TITLE",$obj_shipping->getPrefix(),$ns_bmecat);
            $adress->addChild("bmecat:COUNTRY_CODED",$obj_shipping->getCountryId(),$ns_bmecat);
            $adress->addChild("bmecat:PHONE",$obj_shipping->getTelephone(),$ns_bmecat);
            $adress->addChild("bmecat:FAX",$obj_shipping->getFax(),$ns_bmecat);
            $adress->addChild("bmecat:EMAIL",$obj->getCustomerEmail(),$ns_bmecat);
            $adress->addChild("bmecat:VAT_ID",$obj->getCustomerTaxvat(),$ns_bmecat);
            $street_array=$obj_shipping->getStreet();
            //mehrzeilige adressen zusammenfügen
            if(isset($street_array[1]))
                $street_string=$street_array[0].", ".$street_array[1];    
            else
                $street_string=$street_array[0];     
            $adress->addChild("bmecat:STREET",$street_string,$ns_bmecat);
            $adress->addChild("bmecat:ZIP",$obj_shipping->getPostcode(),$ns_bmecat);
            $adress->addChild("bmecat:CITY",$obj_shipping->getCity(),$ns_bmecat);
            
            $order_info->addChild("CURRENCY",$obj->getOrderCurrencyCode(),$ns_bmecat); //währung der bestellung
            
            //hier wird der Bezahltyp aus dem Onlineshop mitgegeben
            $obj_payment=$obj->getPayment();
            $payment_terms=$order_info->addChild("PAYMENT")->addChild("PAYMENT_TERMS");
            $payment_term=$payment_terms->addChild("PAYMENT_TERM",$obj_payment->getMethod());
            $payment_term->addAttribute("type","other");                
            
            //darstellung des porto damit gesamtsumme stimmt
            $order_zusatz_porto=$payment_terms->addChild("TIME_FOR_PAYMENT")->addChild("ALLOW_OR_CHARGE_FIX")->addChild("ALLOW_OR_CHARGE");
            $order_zusatz_porto->addAttribute("type","surcharge"); //typ zuschlag
            $order_zusatz_porto->addChild("ALLOW_OR_CHARGE_NAME",$obj->getShippingMethod()); //name der zusatzposition
            $order_zusatz_porto->addChild("ALLOW_OR_CHARGE_DESCR",$obj->getShippingDescription()); //beschreibung
            $order_zusatz_porto->addChild("ALLOW_OR_CHARGE_TYPE","postage"); //porto
            $order_zusatz_porto_values=$order_zusatz_porto->addChild("ALLOW_OR_CHARGE_VALUE");
            $order_zusatz_porto_values->addChild("AOC_MONETARY_AMOUNT",$obj->getShippingInclTax()); //portokosten
            $order_zusatz_porto->addChild("ALLOW_OR_CHARGES_TOTAL_AMOUNT",$obj->getShippingInclTax()); //summe zusatzkosten
            
            //bestellpositionen
            $order_item_list=$order->addChild("ORDER_ITEM_LIST");
            
            //lies order_items zu order und gehe über alle
            $item_counter=0;
            foreach ($obj->getItemsCollection() as $item)
            {
                $item_counter++;
                
                //wenn debug aktiv schreibe objekte in files
                if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                    Heliumv_Synchronization_Helper_Master::debugLog('order'.$obj->getId().'_item'.$item_counter.'.txt',$item,"no",gmdate("Ymd"));
                    
                //lies externe_pid aus zusätzlichem object aus    
                $external_pid = Mage::getModel('catalog/product')->load($item->getProductId())->getExternalPid();
                
                //wenn external_id nicht gefunden verwuche external id über sk zu finden
                if($external_pid==null)
                {
                    $external_pid = Mage::getModel('catalog/product')->loadByAttribute('sku',$item->getSku())->getExternalPid();    
                    if($external_pid==null)
                        mage::log("createOrderXML - OrderId:".$obj->getId." konnte zu ArtikelID:".$item->getProductId()." keine external_Pid ermittelt werden (vermutlich produkt geloescht), es wird leer übermittelt",Zend_Log::ERR,self::$logfile_name);
                    else
                        mage::log("createOrderXML - OrderId:".$obj->getId." konnte zu ArtikelID:".$item->getProductId()." direkt keine external_Pid ermittelt werden, es wuerde ueber die SKU:".$item->getSku()." ein Artikel gefunden und dazu die folgende ExternePID:".$external_pid." ermittelt.",Zend_Log::WARN,self::$logfile_name);
                }
                
                //setze order items v2
                $order_item=$order_item_list->addChild("ORDER_ITEM");
                $order_item->addChild("LINE_ITEM_ID",$item_counter);
                $product_id=$order_item->addChild("PRODUCT_ID");
                $product_id->addChild("bmecat:SUPPLIER_PID",$external_pid,$ns_bmecat)->addAttribute("type","supplier_specific");
                $product_id->addChild("bmecat:BUYER_PID",$item->getSku(),$ns_bmecat)->addAttribute("type","buyer_specific");
                $product_id->addChild("DESCRIPTION_SHORT",$item->getName(),$ns_bmecat);
                $order_item->addChild("QUANTITY",$item->getQtyOrdered());
                $order_item->addChild("bmecat:ORDER_UNIT",$item->getOrderUnit,$ns_bmecat); //standardwert da pflichtfeld
                
                //preis pro position
                $product_price_fix=$order_item->addChild("PRODUCT_PRICE_FIX");
                $product_price_fix->addChild("bmecat:PRICE_AMOUNT",$item->getPrice(),$ns_bmecat); //einzelpreis
                
                //steuern pro position
                $tax_details_fix=$product_price_fix->addChild("TAX_DETAILS_FIX");
                $tax_details_fix->addChild("TAX_AMOUNT",$item->getTaxAmount());
                $tax_details_fix->addChild("TAX_BASE",$item->getPrice()); //nettopreis
                $steuersatz_berechnet=floatval($item->getTaxPercent());
                if($steuersatz_berechnet!=0)
                    $steuersatz_berechnet=number_format(($steuersatz_berechnet/100),4);
                else
                    $steuersatz_berechnet="";
                $tax_details_fix->addChild("bmecat:TAX",$steuersatz_berechnet,$ns_bmecat); //steuersatz 0.10=10%, 0.20=20% ...               
                
                //zeilensumme
                $order_item->addChild("PRICE_LINE_AMOUNT",$item->getRowTotal());
            }

            //bestellsummen
            $order_summary=$order->addChild("ORDER_SUMMARY");
            $order_summary->addChild("TOTAL_ITEM_NUM",$item_counter);
            $order_summary->addChild("TOTAL_AMOUNT",$obj->getGrandTotal());


            //wenn debug aktiv schreibe xml in file
            if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                Heliumv_Synchronization_Helper_Master::debugLog('order'.$obj->getId().'.xml',$xml->asXML(),"no",gmdate("Ymd"));
        }
        catch (Exception $ex)
        {
            Mage::log("Fehler bei XML Erstellung für OrderID ".$obj->getId()." aufgetreten, Exception:".$ex->getMessage(),zend_log::ERR,self::$logfile_name);    
        }
        
        return $xml;        
    }
    
    
    /**
    * Prüft laut Einstellung alle X Minuten ob Bestellübertragungen hängen und stoßt diese erneut an, wenn Zeit-Limit erreicht ist wird per eMail alamiert
    */
    public function checkOrderRetry()
    {
        $ts_order_retry=(int)Mage::getStoreConfig('synchronization/ordertools/order_retry_time')*60; //konvertiere settings in sekunden
        $ts_order_retry_max=(int)Mage::getStoreConfig('synchronization/ordertools/order_retry_time_max')*60;    
        
        //setze initialwert von 1min
        if($ts_order_retry<60)
            $ts_order_retry=60;
            
        //setze initialwert von 1h
        if($ts_order_retry_max<3600)
            $ts_order_retry_max=3600;
        
        //gehe durch alle Orders die aktuell Status transfer_start haben oder die einmal laut history transfer_start hatten und jetzt nichts mit transfer_* haben
        $resource = Mage::getSingleton('core/resource');
        $sql="SELECT entity_id,status,state,updated_at,created_at FROM ".$resource->getTableName("sales_flat_order")." where status='".self::STATUS_TRANSFER_START."' ";
        $sql.=" union SELECT entity_id,status,state,updated_at,created_at FROM ".$resource->getTableName("sales_flat_order")." where status not in ('".self::STATUS_TRANSFER_START."','".self::STATUS_TRANSFER_COMPLETE."','".self::STATUS_TRANSFER_ERROR."') and entity_id in (SELECT parent_id FROM ".$resource->getTableName("sales_flat_order_status_history")." where status='".self::STATUS_TRANSFER_START."')";
        $db_read = $resource->getConnection('core_read');
        $sqlresult = $db_read->fetchAll($sql);
        foreach ($sqlresult as $order)
        {
            try
            {
                $order_id=$order["entity_id"];
                $ts_updated=strtotime($order["updated_at"]);
                $ts_created=strtotime($order["created_at"]);
                $ts_now=time();

                //debug log
                if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                {
                    $readable_string="Zeiten fuer Retry OrderID:".$order_id.": ts_updated:".gmdate("d.m.Y H:i:s",$ts_updated).",ts_created:".gmdate("d.m.Y H:i:s",$ts_created).",jetzt:".gmdate("d.m.Y H:i:s").",retry_in_sek:".(string)$ts_order_retry.",retry_until_in_sek:".(string)$ts_order_retry_max."";
                    mage::log($readable_string,null,self::$logfile_name);
                    
                    mage::log("Retry? - da jetzt:".(string)$ts_now." weniger order_update:".(string)$ts_updated." ist ".(string)($ts_now-$ts_updated)." und damit wenn > ".(string)$ts_order_retry." -> retry setzen",null,self::$logfile_name);
                    mage::log("Error? - da jetzt:".(string)$ts_now." weniger order_created:".(string)$ts_created." ist ".(string)($ts_now-$ts_created)." und damit wenn > ".(string)$ts_order_retry_max." -> error setzen",null,self::$logfile_name);
                }
                
                //prüfe ob error status erreicht
                if(($ts_now-$ts_created)>$ts_order_retry_max)
                {
                    //setze error status
                    $return_statuschange=$this->setOrderStatusAndState(Mage::getModel('sales/order')->load($order_id),self::STATUS_TRANSFER_ERROR);
                    
                    //sende error mail
                    if($return_statuschange==true)
                    {
                        mage::log("checkOrderRetry - Give up retry OrderSync for OrderID:".$order_id." + Errormail",zend_log::ERR,self::$logfile_name);
                        Heliumv_Synchronization_Helper_Master::sendErrorMail("Magento: Give up retry OrderSync for OrderID:".$order_id."","The OrderSync retry for OrderID:".$order_id." is cancled now. The Order is not transfered to the ERP Systeme. Please check Order manually in your Magento Onlineshop.");
                    }
                    
                }
                elseif(($ts_now-$ts_updated)>$ts_order_retry) //prüfe ob retry gesetzt werden soll
                {   
                    //setze status auf initialstatus zurück, dadurch wird die order erneut übertragen
                    $return_statuschange=$this->setOrderStatusAndState(Mage::getModel('sales/order')->load($order_id),self::STATUS_NEW);
                    if($return_statuschange==true)
                        mage::log("checkOrderRetry - OrderID:".$order_id." wurde zurueckgesetzt -> Ubertragung wird beim naechsten Mal erneut versucht",zend_log::WARN,self::$logfile_name);  
                }
            }
            catch (Exception $ex)
            {
                Mage::log("checkOrderRetry - Fehler bei OrderRetry Verarbeitung aufgetreten, Daten:".$readable_string." Exception:".$ex->getMessage(),zend_log::ERR,self::$logfile_name);
            }
            
        } //foreach
        
    
        
    }
    
    /**
    * Geht durch alle Orders die noch nie Syncronisiert wurden aber einen anderen Status haben und schicke Info an HV
    * @return true wenn ok und null bei fehler
    */
    public function checkAllOtherOrders()
    {   
        try
        {
            //gehe durch alle Orders die laut history noch nie transfer_start hatten und jetzt nichts mit transfer_* oder neu haben und älter als 1h sind und noch keine ext_order_id haben
            $resource = Mage::getSingleton('core/resource');
            $time_utc_vor_1h=gmdate("Y-m-d H:i:s",(time()-3600));
            $sql="SELECT entity_id,status,state,updated_at,created_at FROM ".$resource->getTableName("sales_flat_order")." where status not in ('".self::STATUS_TRANSFER_START."','".self::STATUS_TRANSFER_COMPLETE."','".self::STATUS_TRANSFER_ERROR."','".self::STATUS_NEW."') and entity_id not in (SELECT parent_id FROM ".$resource->getTableName("sales_flat_order_status_history")." where status='".self::STATUS_TRANSFER_START."') and ext_order_id is null and created_at<'".$time_utc_vor_1h."'";
            $db_read = $resource->getConnection('core_read');
            $sqlresult = $db_read->fetchAll($sql);
            foreach ($sqlresult as $order)
            {
                $order_id=$order["entity_id"];
                $order_status=$order["status"];
                $order_state=$order["state"];
                
                //sende error an HV
                $hv_error_ret=Heliumv_Synchronization_Helper_Master::sendErrorToHV(409,"OrderID:".$order_id." wurde aufgrund von Status/State: ".$order_status."/".$order_state." nicht in der Uebermittlung an HV beruecksichtigt.",Zend_log::ERR,"order",$order_id);
                
                //wenn erfolgreich an HV übermittelt setze ext_order_id auf "error"
                if($hv_error_ret==true)
                {
                    $order_obj = Mage::getModel('sales/order')->load($order_id);
                    $order_obj->setData("ext_order_id","error");
                    $order_obj->save();
                    mage::log("checkAllOtherOrders - ext_order_id von OrderID:".$order_id." wurde auf error gesetzt.",zend_log::ERR,self::$logfile_name);
                }
                    
            }
            return true;
        
        }
        catch(Exception $e)
        {
            mage::log("checkAllOtherOrders - Exception".$e->getMessage(),zend_log::ERR,self::$logfile_name);
            return null;
        }
                        
    }
    
    


}

?>