<?php
/**
* HELIUM V Synchronisation Modul for Magento 1.7.x
*
* Copyright (C) 2013 HELIUM V IT-Solutions GmbH
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU Affero General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* According to sec. 7 of the GNU Affero General Public License, version 3,
* the terms of the AGPL are supplemented with the following terms:
*
* "Helium V" and "Helium 5" are registered trademarks of HELIUM V IT-Solutions GmbH.
* The licensing of the Program under the AGPL does not imply a trademark license.
* Therefore any rights, title and interest in our trademarks remain entirely with us.
* If you want to propagate modified versions of the Program under the
* name "Helium V" or "Helium 5", you may only do so if you have a written permission
* by Helium V IT-Solutions GmbH (to acquire a permission please contact
* Helium V IT-Solutions at trademark@heliumv.com).
*      
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Affero General Public License for more details.
*
* You should have received a copy of the GNU Affero General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
* 
* @category Heliumv
* @package  Heliumv_Synchronization
* @author   Maximilian Hehenwarter <developers@heliumv.com>
* @license  http://www.gnu.org/licenses/  Affero General Public License (AGPL 3.0) 
*/

/**
* Diese Klasse stellt eine Multithread funktionalität für den Aufruf von Magento Classen / mit Funktionen dar
* Voraussetzung dafür ist, dass im Magento Cronjob jede Minute der Job Heliumv_Synchronization_Model_Multithread::startThread aufgerufen wird
* Info und Beispiele von Handbuch p63ff
*/
class Heliumv_Synchronization_Model_Scheduler extends Mage_Core_Model_Abstract
{
    //definiere statuswerte hier
    const STATUS_NEW  = 1;
    const STATUS_WORK  = 2;
    const STATUS_FINISHED  = 3;
    const STATUS_ERROR  = 4;
    const STATUS_IGNORPRIO  = 5;
    const STATUS_RESERVEJOBNR  = 6;
    
    /** variablendeklaration logfile für scheduler aufrufe */
    private static $logfile_name="hv_scheduler_thread.log";
    /** variablendeklaration logfile für scheduler element neuerstellung */
    private static $create_logfile_name="hv_scheduler_create.log";
    /** variablendeklaration logfile für die einzeknen scheduler threads */
    private static $job_logfile_name="hv_scheduler_job.log";
    /** variablendeklaration aktuelle scheduler thread job nr */
    public $current_job_nr="";
    
    /**
     * In Magento Models darf der Ã¼bliche PHP Konstruktor __construct() nicht
     * Ã¼berlagert werden, statt dessen kann als Alternative die Methode
     * _contruct() verwendet werden.
     */
    protected function _construct()
    {
        // Name des zu diesem Models gehÃ¶rigen Resource Models
        $this->_init('synchronization/scheduler');
        
        //immer änderungsdatum setzen wenn objekt initialisiert wird
        $this->setData("ts_changed",strtotime(now()));
    }
    
    
    /**
    * Scheduler wird vom Cronjob laufend angestoßen
    */
    public function runScheduler()
    {
        //scheduler laut konfig aktiv
        if(Heliumv_Synchronization_Helper_Master::getAsyncFlag()==0)
        {
            if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                mage::log("Scheduler in Konfig nicht aktiviert -> Abbruch runScheduler",null,self::$job_logfile_name);
            return null;
        }
            
        
        ///gibt es scheduler elemente in status new (fehler werden auch auf new zurück gesetzt)
        $search_new_data=$this->load(self::STATUS_NEW,"status");
        if($search_new_data->getId()==null)
        {
            if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                mage::log("Keine Schedulerelemente mit Status New gefunden -> Abbruch runScheduler",null,self::$job_logfile_name);
            return null;
        }
        
        //wenn Helium nicht erreichbar abbruch
        $ping=Heliumv_Synchronization_Helper_Master::doPingHV();
        if($ping!=1)
            return null;

        
        //laufen eh weniger als die erlaubten jobs, ermittle neue jobnr
        $new_jobnr=$this->getNewJobNr();

        //wenn jobnr vorhanden starte neuen scheduler thread
        if($new_jobnr!=null)
        {
            //erstelle 2 zusätzlichen cronjobs jobs im abstand von 10sek um eine skalierung nach oben zu ermöglichen
            try
            {
                //etwas heikel -> wieder auf 1 reduziert da sofort skalierung sonnst sofort nach oben geht
                $resource = Mage::getSingleton('core/resource');
                $db_write = $resource->getConnection('core_write');
                $plus_sek=10;
                $sql="INSERT INTO cron_schedule (job_code,status,messages,created_at,scheduled_at,executed_at,finished_at) VALUES ('syncronisation_scheduler','pending',null,{ts '".gmdate("Y-m-d H:i:s")."'},{ts '".gmdate("Y-m-d H:i:s",(time()+$plus_sek))."'},null,null)";
                $db_write->query($sql);
                $plus_sek=20;
                //$db_write->query($sql);
                mage::log("Erstelle 1 weitere Cronjobs fuer Schedulerstart",zend_log::INFO,self::$job_logfile_name);
            }
            catch(Exception $ex)
            {
                mage::log("Fehler bei Erstellung zusaetzlicher Cronjobs aufgetreten".$ex->getMessage(),zend_log::ERR,self::$job_logfile_name);
            }
            
            //dana fürhre thread aus
            $this->runSchedulerThread($new_jobnr);
        }
        else
        {
            return null;
        }    
    }
    
    /**
    * Erstellt neues Scheduler Element für asyncronen Aufruf aus Cronjob
    * 
    * @param string $class - Classe die initialisiert werden soll
    * @param string $call - function in der classe die aufgerufen
    * @param array or string $arguments -  array von parametern die der funktion übergeben werden sollen, maximal 5 nummerische werte erlaubt (wird serialisiert in db gespeichert)
    * @param int $prio - Priorität der Verarbeitung startet mit 1, default=3
    * @return - liefert entity-id zurück, im Fehlerfall null
    */
    public function createSchedulerElement($class,$call,$arguments,$prio=3)
    {
        //es werden maximal 5 nummerische parameter für den functionsaufruf unterstützt
        if(isset($arguments[5]))
        {
            Mage::log("Es wurden mehr als 5 Parameter übergeben, Speichern von SchedulerElement fehlgeschlagen (".$class."->".$call."(".$arguments."))",zend_log::ERR,self::$create_logfile_name); 
            return null;  
        }

        //prüfe ob class vorhanden ist
        if(class_exists($class)==false)
        {
            mage::log("SchedulerElement Anlage fehlgeschlagen, Klasse (".$class.") nicht vorhanden.",zend_log::ERR,self::$create_logfile_name);
            return null;
        }        
        //prüfe ob funktion vorhanden ist
        if(method_exists($class,$call)==false)
        {
            mage::log("SchedulerElement Anlage fehlgeschlagen, Methode (".$call.") nicht vorhanden.",zend_log::ERR,self::$create_logfile_name);
            return null;
        }
        
        
        $this->setData("status",self::STATUS_NEW); //1=new
        //$this->setData("jobnr",$jobnr); //default =0, wird manuell nur für reservierung von jobs gesetzt
        $this->setData("class",$class);
        $this->setData("call",$call);
        $this->setData("arguments",serialize($arguments));
        $this->setData("prio",$prio);
        
        try
        {
            $this->save();
            if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                mage::log("SchedulerElement mit ID:".$this->getId()." und Call:".$call." angelegt.",zend_log::INFO,self::$create_logfile_name);
            return $this->getId();  
        }
        catch(Exception $ex)
        {
            Mage::log("Exception bei Speichern von SchedulerElement ausgetreten (".$class."->".$call."(".print_r($arguments,1).")), Exception:".$ex->getMessage(),zend_log::ERR,self::$create_logfile_name); 
            return null;   
        }
        
    }
    
    /**
    * Ermittelt und Reserviert neue Jobnr für Scheduler Thread, wird aufgerufen aus run scheduler, verwendet lock scheduler_jobnr um eine gleichzeitoige vergabe der selben nummer zu unterbinden
    * Löscht zusätzlich alte Jobnummern welche seit $max_thread_runtime (zB 10min) nicht mehr aktualisiert wurden
    * @return - liefert neue jobnr oder wenn es keine Gibt oder im Fehlerfall null zurück
    */
    private function getNewJobNr()
    {
        $return = null;
        
        //jobnummern vergabe nur für einen prozess auf einmal möglich
        $lock = new Mage_Index_Model_Process();
        $lock->setId("scheduler_jobnr");
        
        try
        {
            //setze lock bzw warte auf nächsten freien lock
            $save_start=microtime(1);
            $lock->lockAndBlock();
            if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                mage::log("Dauer getlock scheduler_jobn: ".number_format(microtime(1)-$save_start,4),null,self::$job_logfile_name); 
            
            //einstellung aus config lesen - default sind 10 jobs  
            $max_allowed_jobs=Mage::getStoreConfig('synchronization/scheduler/max_parallel_threads');
            if($max_allowed_jobs==null || $max_allowed_jobs=="" || $max_allowed_jobs==0)
                $max_allowed_jobs=10; 
                
            //maximallaufzeit eines jobs auslesen - default ist 600sek = 10min
            $max_thread_runtime=Mage::getStoreConfig('synchronization/scheduler/max_thread_runtime');
            if($max_thread_runtime==null || $max_thread_runtime=="" || $max_thread_runtime==0)
                $max_thread_runtime=600; 
                
                
            //entferne alle jobnummern welche seit maximallaufzeit von thread nicht mehr aktualisiert wurden
            $ts_string=gmdate("Y-m-d H:i:s",(time()-$max_thread_runtime));
            $resource = Mage::getSingleton('core/resource');
            $sql="SELECT entity_id,jobnr FROM ".$resource->getTableName("heliumv_synchronization_scheduler")." where status=".self::STATUS_RESERVEJOBNR." and ts_changed<'".$ts_string."'";
            $db_read = $resource->getConnection('core_read');
            $sqlresult = $db_read->fetchAll($sql); 
            if(isset($sqlresult) && isset($sqlresult[0]))
            {
                foreach($sqlresult as $element)
                {
                    $entity_id=$element["entity_id"];
                    $this->load($entity_id);
                    $this->delete();
                    if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                        mage::log("JobNr ".$element["jobnr"]." Reservierung aus Entity_ID:".$entity_id." geloescht da TS aelter als ".$max_thread_runtime."sek.",null,self::$job_logfile_name); 
                
                }
            }
            
            //sql um alle vorhandenen jobids abzufragen
            $sql="SELECT distinct count(distinct jobnr) as anzahl_jobsnr,max(jobnr) as max_jobnr FROM ".$resource->getTableName("heliumv_synchronization_scheduler")." where status=".self::STATUS_RESERVEJOBNR."";
            $db_read = $resource->getConnection('core_read');
            $sqlresult = $db_read->fetchRow($sql); 

            //wenn noch nicht die maximale anzahl an jobs erreicht ist
            if($sqlresult["anzahl_jobsnr"]<$max_allowed_jobs)
            {
                //berechne neue jobnummer    
                $new_jobnr=(int)$sqlresult["max_jobnr"]+1;
                
                //erstelle scheduler eintrag mit speziellem status um die jobnummer zu reservieren
                $new_sched_element=new Heliumv_Synchronization_Model_Scheduler();
                $new_sched_element->setData("status",self::STATUS_RESERVEJOBNR); 
                $new_sched_element->setData("jobnr",$new_jobnr);
                //$new_sched_element->setData("prio",99);
                $new_sched_element->save();
                
                if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                    mage::log("JobNr:".$new_jobnr." fuer SchedulerThread in SchedulerElementID:".$new_sched_element->getId()." reserviert.",zend_log::DEBUG,self::$job_logfile_name);
                $return = $new_jobnr;
            }
            else
            {
                //maximalanzahl an jobs erreicht
                mage::log("Keine JobNr vergeben da ".$sqlresult["anzahl_jobsnr"]." Jobs laufen und laut Konfig nur ".$max_allowed_jobs." erlaubt sind.",zend_log::WARN,self::$job_logfile_name);
                $return=null;
            }
            
            //beende lock
            $lock->unlock();
        }
        catch(Exception $ex)
        {
            //beende lock
            $lock->unlock();
            Mage::log("Exception bei Ermittlung neue JobNr aufgetreten, Exception:".$ex->getMessage(),zend_log::ERR,self::$job_logfile_name); 
            $return= null;   
        }
                
        return $return;
        
    }
    
    /**
    * Aktualisiert TS für Vergabe der Jobnummer, meldet sich ein Job X Minuten nicht wird er automatisch wieder vergeben
    * @param integer $job_nr - welcher job soll refreshed werden?
    * @return - wenn OK 1, im fehlerfall null
    */
    private function refreshJobNr($job_nr)
    {
        $return=null;
        try
        {
            //ermittle element
            $resource = Mage::getSingleton('core/resource');
            $sql="SELECT entity_id FROM ".$resource->getTableName("heliumv_synchronization_scheduler")." where jobnr=".$job_nr." and status=".self::STATUS_RESERVEJOBNR."";
            $db_read = $resource->getConnection('core_read');
            $sqlresult = $db_read->fetchRow($sql); 
            $entity_id=$sqlresult["entity_id"];
            
            //aktualisiere ts
            $this->load($entity_id);
            $this->setData("ts_changed",gmdate("Y-m-d H:i:s"));
            $this->save();  
            if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                Mage::log($job_nr." JobNr is alive",zend_log::DEBUG,self::$job_logfile_name); 
            $return = 1;  
        }
        catch(Exception $ex)
        {
            //beende lock
            $lock->unlock();
            Mage::log("Exception bei refreshJobNr für JobNr:".$job_nr." aufgetreten, Exception:".$ex->getMessage(),zend_log::ERR,self::$job_logfile_name); 
            $return= null;   
        }
        return $return;    
    }
    
    /**
    * Ändert den Status und/Oder die JobNr eines SchedulerElements
    * 
    * @param mixed $entity_id  - Pflichtfeld, ID des Scheduler Elements
    * @param mixed $status - Optional, zu setzender Status des Scheduler Elements
    * @param mixed $jobnr - Optional, zu setzende Jobnr des Scheduler Elements
    * @return - bei Erfolg 1, sonnst null
    */
    private function changeSchedulerElementStatus($entity_id,$status=null,$jobnr=null)
    {
        //eines der beiden werte muss übergeben werden
        if($status==null && $jobnr=null)
            return null;
        
        $return=null;
        
        try
        {
            //ändere status von scheduler element
            $this->load($entity_id);
            if($status!=null)
                $this->setData("status",$status);
            if($jobnr!=null)
                $this->setData("jobnr",$jobnr);  
            $this->save();
            if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                mage::log($this->current_job_nr." - Aenderung(Status=".$status.",Jobnr=".$jobnr.") von SchedulerElement(".$entity_id.") OK.",zend_log::DEBUG,self::$logfile_name);
            $return = 1;

        }
        catch(Exception $ex)
        {
            Mage::log("Errow bei Aenderung(Status=".$status.",Jobnr=".$jobnr.") von SchedulerElement(".$entity_id."), Exception:".$ex->getMessage(),zend_log::ERR,self::$logfile_name); 
            $return= null;   
        }
        
        return $return;
        
    }
    
    /**
    * Löscht Scheduler Element
    * 
    * @param mixed $entity_id  - Pflichtfeld, ID des Scheduler Elements
    * @return - bei Erfolg 1, sonnst null
    */
    private function deleteSchedulerElement($entity_id)
    {
        $return=null;
        try
        {
            $this->load($entity_id);
            $this->delete();
            if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                mage::log($this->current_job_nr." - Verarbeitung OK - SchedulerElement(".$entity_id.") geloescht.",zend_log::INFO,self::$logfile_name);
            $return = 1;
        }
        catch(Exception $ex)
        {
            Mage::log($this->current_job_nr." - Fehler beim Loeschen von SchedulerElement(".$entity_id."), Exception:".$ex->getMessage(),zend_log::ERR,self::$logfile_name); 
            $return= null;   
        }
        return $return;
    }
    
    /**
    * Sucht nach einer Jobnr-Reservierung und Löscht diese (Scheduler Element status 6)
    * 
    * @param mixed $jobnr - die jeweilige jobnummer die gelöscht werden soll
    * @return - bei Erfolg 1, sonnst null
    */
    private function deleteJobnumberReservation($jobnr)
    {
        $status=self::STATUS_RESERVEJOBNR;
        $return=null;
        
        //jobnummern vergabe und löschung nur für einen prozess auf einmal möglich
        $lock = new Mage_Index_Model_Process();
        $lock->setId("scheduler_jobnr");
        
        try
        {
            //setze lock bzw warte auf nächsten freien lock
            $save_start=microtime(1);
            $lock->lockAndBlock();
            if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                mage::log("deleteJobnumberReservation - Dauer getlock scheduler_jobn: ".number_format(microtime(1)-$save_start,4),null,self::$job_logfile_name); 
            
            
            //suche alle in frage kommenden elemente via sql
            $resource = Mage::getSingleton('core/resource');
            $sql="SELECT entity_id FROM ".$resource->getTableName("heliumv_synchronization_scheduler")." where jobnr=".$jobnr." and status=".$status."";
            $db_read = $resource->getConnection('core_read');
            $sqlresult = $db_read->fetchAll($sql); 
            
            foreach($sqlresult as $element)
            {
                $entity_id=$element["entity_id"];
                $this->load($entity_id);
                $this->delete();
                if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                    mage::log($this->current_job_nr." - SchedulerElement(".$entity_id.") mit Jobnr=".$jobnr.",Status=".$status." geloescht.",zend_log::INFO,self::$job_logfile_name);
            }
            $return = 1;
            
            //beende lock
            $lock->unlock();
            
        }
        catch(Exception $ex)
        {
            Mage::log($this->current_job_nr." - Fehler beim Loeschen von allen SchedulerElementen mit Jobnr=".$jobnr.",Status=".$status." aufgetreten, Exception:".$ex->getMessage(),zend_log::ERR,self::$logfile_name); 
            $return= null; 
            
            //beende lock
            $lock->unlock();  
        }
        return $return;
    }
    
    /**
    * Sucht nach Elementen mit Status ungleich new und älter als X Tage und löscht diese Scheduler Elemente
    * 
    * @param mixed $days_old - wieviele Tage müssen die Elemente mindestens sein damit sie gelöscht werden, default=5
    * @return - bei Erfolg 1, sonnst null
    */
    private function deleteOldSchedulerElements($days_old=5)
    {
        $return=null;
        try
        {
            //ermittle zeitstempel aus parameter
            $ts=time()-((int)$days_old*24*60*60);
            $ts_string=gmdate("Y-m-d H:i:s",$ts);
            
            //suche alle in frage kommenden elemente via sql
            $resource = Mage::getSingleton('core/resource');
            $sql="SELECT entity_id,ts_changed FROM ".$resource->getTableName("heliumv_synchronization_scheduler")." where status<>".self::STATUS_NEW." and ts_changed<'".$ts_string."'";
            $db_read = $resource->getConnection('core_read');
            $sqlresult = $db_read->fetchAll($sql); 
            
            foreach($sqlresult as $element)
            {
                $entity_id=$element["entity_id"];
                $this->load($entity_id);
                $this->delete();
                mage::log($this->current_job_nr." - SchedulerElement(".$entity_id.") gelöscht weil ts_changed(".$element["ts_changed"].") aelter als (".$ts_string."), beide in UTC.",zend_log::INFO,self::$logfile_name);
            }
            $return = 1;
        }
        catch(Exception $ex)
        {
            Mage::log($this->current_job_nr." - Fehler beim Loeschen von allen SchedulerElementen älter als UTC ".$ts_string." aufgetreten, Exception:".$ex->getMessage(),zend_log::ERR,self::$logfile_name); 
            $return= null;   
        }
        return $return;
    }
    
    /**
    * Verarbeite Errorcounter, erhöhe Error Counter um 1
    * - Maximal Retrys nicht erreicht -> setze Status auf new um Element erneut zu verarbeiten
    * - Maximal Retrys erreicht -> setze Status auf error
    * 
    * @param mixed $entity_id  - Pflichtfeld, ID des Scheduler Elements
    * @return - wenn OK 1, bei Error null
    */
    private function checkErrorCounter($entity_id)
    {
        $return=null;
        
        try
        {
            //maximaler error retry counter aus config - default ist 3
            $max_retry_counter=Mage::getStoreConfig('synchronization/scheduler/max_retry_counter');
            if($max_retry_counter==null || $max_retry_counter=="" || $max_retry_counter==0)
                $max_retry_counter=3; 
            
            //prüfe errorcounter
            $this->load($entity_id);
            $retry_counter=$this->getData("retry_counter");
            
            //versuche erneuten retry -> retry ++ und status new
            if($retry_counter<$max_retry_counter)
            {
                $retry_counter_new=(int)$retry_counter+1;
                $this->setData("retry_counter",$retry_counter_new);
                $this->setData("status",self::STATUS_NEW);    
                mage::log($this->current_job_nr." - Aenderung (Status=new,RetryCounter=".$retry_counter_new.") von SchedulerElement(".$entity_id.").",zend_log::WARN,self::$logfile_name);    
            }
            else
            {
                //setze status auf error
                $this->setData("status",self::STATUS_ERROR);  
                mage::log($this->current_job_nr." - Aenderung (Status=error) von SchedulerElement(".$entity_id.").",zend_log::ERR,self::$logfile_name);  
            }
            
            //element speichern
            $this->save();
            $return = 1;
        }
        catch(Exception $ex)
        {
            Mage::log($this->current_job_nr." - Errow bei checkErrorCounter von SchedulerElement(".$entity_id."), Exception:".$ex->getMessage(),zend_log::ERR,self::$logfile_name); 
            $return= null;   
        }
        
        return $return;
        
    }
    
    
    /**
    * Läuft so lange bis maximale Thread Laufzeit aus ist oder niochts mehr zu tunm ist
    * @param $job_nr - mit welcher id läuft der thread
    */
    public function runSchedulerThread($job_nr)
    {
        //setze jobnummer in aktuellem object
        $this->current_job_nr=$job_nr;
        
        //lese maximale schedulerlaufzeit aus config
        $max_thread_runtime=Mage::getStoreConfig('synchronization/scheduler/max_thread_runtime');
        if($max_thread_runtime==null || $max_thread_runtime=="" || $max_thread_runtime==0)
            $max_thread_runtime=600; //fehlerhandling - default sind 600 sekunden laufzeit pro thread 
            
        
        //lese ignor_prio_limit aus config, so lanmge wird gewartet bis ein new gestartet wird trotz ein work mit niedrigerer prio da ist
        $ignor_prio_limit=Mage::getStoreConfig('synchronization/scheduler/ignor_prio_limit');
        if($ignor_prio_limit==null || $ignor_prio_limit=="" || $ignor_prio_limit==0)
            $ignor_prio_limit=120; //fehlerhandling - default sind 120 sekunden 
        
        //startzeit wegschreiben
        $startzeit=time();
        $endzeit=$startzeit+(int)$max_thread_runtime;
        
        mage::log($job_nr." - starte neuen scheduler thread mit nr ".$job_nr.": Endzeit als UTC ist:".gmdate("d.m.Y H:i:s",$endzeit),zend_log::INFO,self::$job_logfile_name); 
        
        //create new lock checking objeck
        $lock = new Mage_Index_Model_Process();
        $lock->setId("scheduler_thread");
        
        //create sql ressource
        $resource = Mage::getSingleton('core/resource');
        
        //laufe so lange bis timeout erreicht ist oder keine daten mehr gefunden werden
        while($endzeit>time())
        {
        
            //locking mechanissmus aus magento verwenden, siehe auch http://www.alexanderinteractive.com/blog/2012/08/lock-your-own-processes-in-magento/
            try
            {
                //setze lock bzw warte auf nächsten freien lock
                $save_start=microtime(1);
                $lock->lockAndBlock();
                if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                    mage::log($job_nr." - Dauer getlock scheduler_thread:".number_format(microtime(1)-$save_start,4),null,self::$logfile_name); 
                
                //suche new element mit kleinster prio und working element mit kleinster prio
                //$sql="SELECT 'new' as type,a.* FROM ".$resource->getTableName("heliumv_synchronization_scheduler")." a where status=".self::STATUS_NEW." group by prio having prio=min(prio) limit 1 union select 'work' as type,b.* from ".$resource->getTableName("heliumv_synchronization_scheduler")." b where status=".self::STATUS_WORK." group by prio having prio=min(prio) limit 2";
                //suche new element mit kleinster prio und working element mit kleinster prio und wo retry_flag am kleinsten ist
                $sql="SELECT 'new' as type,a.* FROM ".$resource->getTableName("heliumv_synchronization_scheduler")." a where status=".self::STATUS_NEW." group by prio,retry_counter having prio=min(prio) and retry_counter=min(retry_counter) limit 1 union select 'work' as type,b.* from ".$resource->getTableName("heliumv_synchronization_scheduler")." b where status=".self::STATUS_WORK." group by prio,retry_counter having prio=min(prio) and retry_counter=min(retry_counter) limit 2";
                $db_read = $resource->getConnection('core_read');
                $sqlresult = $db_read->fetchAll($sql); 
                
                //element mit status neu vorhanden
                if(isset($sqlresult[0]["type"]) && $sqlresult[0]["type"]=="new" && isset($sqlresult[0]["prio"]) && isset($sqlresult[0]["entity_id"]))
                {
                    
                    //element mit status work vorhanden 
                    if(isset($sqlresult[1]["type"]) && $sqlresult[1]["type"]=="work"  && isset($sqlresult[1]["prio"]) && isset($sqlresult[1]["entity_id"]))
                    {
                        //initialisiere prios
                        if(is_numeric($sqlresult[1]["prio"]))
                            $prio_work=(int)$sqlresult[1]["prio"];
                        else
                            $prio_work=99;
                        if(is_numeric($sqlresult[0]["prio"]))
                            $prio_new=(int)$sqlresult[0]["prio"];
                        else
                            $prio_new=99;
                            
                        //hat work element eine kleinere prio als das new element
                        if($prio_work<$prio_new)
                        {
                            if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                                mage::log($job_nr." - Prio Work kleiner als Prio New, Priolimit=".$ignor_prio_limit."sek, change_ts=".$sqlresult[1]["ts_changed"]." time=".time()." strtotime change_ts".strtotime($sqlresult[1]["ts_changed"]),null,self::$logfile_name); 
                            
                            //íst die letzte änderung des work objects länger her als der ignor prio ts -> setze status auf ignore_prio
                            if((strtotime($sqlresult[1]["ts_changed"])+$ignor_prio_limit)<time())
                            {
                                $entity_id=$sqlresult[1]["entity_id"];
                                $this->changeSchedulerElementStatus($entity_id,self::STATUS_IGNORPRIO);

                                //release lock
                                $lock->unlock();        
                            }
                            else
                            {
                                //release lock
                                $lock->unlock();        
                                if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                                    mage::log($job_nr." - Prio Work kleiner als Prio New, ignor_prio_ts noch nicht erreicht, warte 5sek",null,self::$logfile_name); 
                                sleep(5);
                            }
                            
                        }
                        else
                        {
                            //setze element auf status work und die jobnr
                            $entity_id=$sqlresult[0]["entity_id"];
                            $this->changeSchedulerElementStatus($entity_id,self::STATUS_WORK,$job_nr);
                            
                            //beende lock
                            $lock->unlock();
                            
                            //starte ausführung der function
                            $return=$this->executeFunction($entity_id);
                            
                            //setze lock bzw warte auf nächsten freien lock
                            $save_start=microtime(1);
                            $lock->lockAndBlock();
                            if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                                mage::log($job_nr." - Dauer getlock scheduler_thread:".number_format(microtime(1)-$save_start,4),null,self::$logfile_name); 
                
                            //fehler in der verarbeitung -> setze errorcounter
                            if($return==null)
                            {   
                                $this->checkErrorCounter($entity_id);
                            }
                            else
                            {
                                $this->deleteSchedulerElement($entity_id);    
                            }
                            
                            //beende lock
                            $lock->unlock();
                        }
                        
                    } //element in status work vorhanden
                    else
                    {
                        //nur neues element gefunden, kein element in status work gefunden
                        
                        //setze element auf status work und die jobnr
                        $entity_id=$sqlresult[0]["entity_id"];
                        $this->changeSchedulerElementStatus($entity_id,self::STATUS_WORK,$job_nr);
                        
                        //beende lock
                        $lock->unlock();
                        
                        //starte ausführung der function
                        $return=$this->executeFunction($entity_id);
                        
                        //setze lock bzw warte auf nächsten freien lock
                        $save_start=microtime(1);
                        $lock->lockAndBlock();
                        if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                            mage::log($job_nr." - Dauer getlock scheduler_thread:".number_format(microtime(1)-$save_start,4),null,self::$logfile_name); 
                
                        //fehler in der verarbeitung -> setze errorcounter
                        if($return==null)
                        {   
                            $this->checkErrorCounter($entity_id);
                        }
                        else
                        {
                            $this->deleteSchedulerElement($entity_id);    
                        }
                        
                        //beende lock
                        $lock->unlock();
                        
                    }

                }
                else
                {
                    //kein neues element mehr gefunden -> beende lock und springe aus schleife -> beendet thread
                    $lock->unlock();
                    if(Heliumv_Synchronization_Helper_Master::getDebugFlag())
                        mage::log($job_nr." - kein element status new gefunden -> leite ende jobnr:".$job_nr." ein.",null,self::$job_logfile_name);
                    $endzeit=(int)time()-1;   //setze zeit damit while abgebrochen wird
                }
            
                //aktualisiere zeitstempel auf jobreservierung, sonnst geht jobnummer verloren
                $this->refreshJobNr($job_nr);
            
            } //try
            catch(Mage_Exception $ex)
            {
                //beende lock
                $lock->unlock();
                Mage::log($job_nr." - Exception bei SchedulerElement Vergabe aufgetreten, Exception:".$ex->getMessage(),zend_log::ERR,self::$logfile_name); 
            }

        } //while laufzeit
        
        //lösche reservierung der jobnummer
        $this->deleteJobnumberReservation($job_nr);
        
        //generelles errorhandling für hängende schedulerelemente, lösche alle nicht neuen elemente älter als 5 tage
        $this->deleteOldSchedulerElements(5);
        
        //verarbeitung zu thread ende
        mage::log($job_nr." - beende runSchedulerThread",zend_log::INFO,self::$job_logfile_name);
    }
    
    /**
    * Führt die externe Funktion aus dem jeweiligen modell aus
    * @param int $entity_id - benötigt die ID des jeweiligen SchedulerElements
    * @return - liefert den Return des jeweiligen Aufrufs zurück oder im fehlerfall null
    */
    private function executeFunction($entity_id)
    {
        
        $save_start=microtime(1);

        //lade scheduler element welches übergeben wurde
        $scheduler_element=$this->load($entity_id);
            
        //daten aus element einlesen
        $class=$scheduler_element->getData("class");
        $call=$scheduler_element->getData("call");
        $job_nr=$scheduler_element->getData("jobnr");
        $arguments=unserialize($scheduler_element->getData("arguments"));
        
        //prüfe ob class vorhanden ist
        if(class_exists($class)==false)
            return null;        
        //prüfe ob funktion vorhanden ist
        if(method_exists($class,$call)==false)
            return null;
            
        try
        {
            //lade klasse mittels magento autoload
            $object = Mage::getModel($class);
            
            //rufe funktion auf mit geweils unterschiedlicher parameteranzahl, siehe auch http://stackoverflow.com/questions/251485/dynamic-class-method-invocation-in-php
            if(!isset($arguments[0]))
                $return=$object->$call(); //kein parameter übergeben
            if(isset($arguments[0]) && !isset($arguments[1]))
                $return=$object->$call($arguments[0]); //nur ein parameter wird übergeben
            if(isset($arguments[0]) && isset($arguments[1]) && !isset($arguments[2]))
                $return=$object->$call($arguments[0],$arguments[1]); //zwei parameter übergeben
            if(isset($arguments[0]) && isset($arguments[1]) && isset($arguments[2]) && !isset($arguments[3]))
                $return=$object->$call($arguments[0],$arguments[1],$arguments[2]); //drei parameter übergeben
            if(isset($arguments[0]) && isset($arguments[1]) && isset($arguments[2]) && isset($arguments[3]) && !isset($arguments[4]))
                $return=$object->$call($arguments[0],$arguments[1],$arguments[2],$arguments[3]); //vier parameter übergeben
            if(isset($arguments[0]) && isset($arguments[1]) && isset($arguments[2]) && isset($arguments[3]) && isset($arguments[4]) && !isset($arguments[5]))
                $return=$object->$call($arguments[0],$arguments[1],$arguments[2],$arguments[3],$arguments[4]); //fünf parameter übergeben
        }
        catch(Exception $ex)
        {
            Mage::log("Exception beim executeFunction aufgetreten  (".$class."->".$call."(".print_r($arguments,1).")), Exception:".$ex->getMessage(),zend_log::ERR,self::$logfile_name); 
            return null;       
        }
        
        //schreibe verarbeitung dauer in log
        if(Heliumv_Synchronization_Helper_Master::getPositiveLogFlag())
            mage::log($job_nr." - executeFunction (".$class."->".$call."(".serialize($arguments).")) Dauer: ".number_format(microtime(1)-$save_start,4),null,self::$logfile_name);             
            
        return $return;
    }
        
    


}